
import sys

from ida_hexrays import *
from ida_allins import *
from ida_auto import *
from ida_bitrange import *
from ida_bytes import *
from ida_dbg import *
from ida_diskio import *
from ida_dirtree import *
from ida_entry import *
from ida_enum import *
from ida_expr import *
from ida_fixup import *
from ida_fpro import *
from ida_frame import *
from ida_funcs import *
from ida_gdl import *
from ida_graph import *
from ida_ida import *
from ida_idaapi import *
from ida_idc import *
from ida_idd import *
from ida_idp import *
from ida_ieee import *
from ida_kernwin import *
from ida_lines import *
from ida_loader import *
from ida_moves import *
from ida_nalt import *
from ida_name import *
from ida_netnode import *
from ida_offset import *
from ida_pro import *
from ida_problems import *
from ida_range import *
from ida_registry import *
from ida_search import *
from ida_segment import *
from ida_segregs import *
from ida_srclang import *
from ida_strlist import *
from ida_struct import *
from ida_tryblks import *
from ida_typeinf import *
from ida_ua import *
from ida_xref import *

# guerilla-patch a few unfortunate overrides
from ida_funcs import set_func_start
from ida_funcs import set_func_end
from ida_dbg import dbg_can_query

class idaapi_Cvar(object):
    def __init__(self):
        # prevent endless recursion
        object.__setattr__(self, "modules", "hexrays,allins,auto,bitrange,bytes,dbg,diskio,dirtree,entry,enum,expr,fixup,fpro,frame,funcs,gdl,graph,ida,idaapi,idc,idd,idp,ieee,kernwin,lines,loader,moves,nalt,name,netnode,offset,pro,problems,range,registry,search,segment,segregs,srclang,strlist,struct,tryblks,typeinf,ua,xref".split(","))
        object.__setattr__(self, "cvars_entries", dict())

    def _get_module_cvar(self, modname):
        mod = sys.modules["ida_%s" % modname]
        cv, entries = None, None
        if hasattr(mod, "cvar"):
            cv = getattr(mod, "cvar")
            entries = []
            if cv:
                if modname in self.cvars_entries.keys():
                    entries = self.cvars_entries[modname]
                else:
                    # Parse 'str' version of cvar. Although this is braindeader than
                    # braindead, I'm not sure there's another way to do it.
                    entries_s = str(cv)
                    entries = entries_s[1:len(entries_s)-1].split(", ")
                    self.cvars_entries[modname] = entries
        return cv, entries

    def __getattr__(self, attr):
        for mod in self.modules:
            cv, entries = self._get_module_cvar(mod)
            if cv and attr in entries:
                return getattr(cv, attr)

    def __setattr__(self, attr, value):
        for mod in self.modules:
            cv, entries = self._get_module_cvar(mod)
            if cv and attr in entries:
                setattr(cv, attr, value)


cvar = idaapi_Cvar()



"""
summary: interactively color certain pseudocode lines

description:
  Provides an action that can be used to dynamically alter the
  lines background rendering for pseudocode listings (as opposed to
  using `ida_hexrays.cfunc_t.pseudocode[N].bgcolor`)

  After running this script, pressing 'M' on a line in a
  "Pseudocode-?" widget, will cause that line to be rendered
  with a special background color.

keywords: colors
"""

import ida_kernwin
import ida_hexrays
import ida_moves
import ida_idaapi

class pseudo_line_t(object):
    def __init__(self, func_ea, line_nr):
        self.func_ea = func_ea
        self.line_nr = line_nr

    def __hash__(self):
        return hash((self.func_ea, self.line_nr))

    def __eq__(self, r):
        return self.func_ea == r.func_ea \
            and self.line_nr == r.line_nr


def _place_to_line_number(p):
    return ida_kernwin.place_t.as_simpleline_place_t(p).n


class pseudocode_lines_rendering_hooks_t(ida_kernwin.UI_Hooks):
    def __init__(self):
        ida_kernwin.UI_Hooks.__init__(self)
        self.marked_lines = {}

    def get_lines_rendering_info(self, out, widget, rin):
        vu = ida_hexrays.get_widget_vdui(widget)
        if vu:
            entry_ea = vu.cfunc.entry_ea
            for section_lines in rin.sections_lines:
                for line in section_lines:
                    coord = pseudo_line_t(
                        entry_ea,
                        _place_to_line_number(line.at))
                    color = self.marked_lines.get(coord, None)
                    if color is not None:
                        e = ida_kernwin.line_rendering_output_entry_t(line)
                        e.bg_color = color
                        out.entries.push_back(e)


class toggle_line_marked_ah_t(ida_kernwin.action_handler_t):

    """
    We could very well use an ARGB value, but instead let's go
    go with a color 'key': those can be altered by the user/theme,
    and therefore have a better chance of being appropriate (or at
    least expected.)
    """
    COLOR_KEY = ida_kernwin.CK_EXTRA11

    def __init__(self, hooks):
        ida_kernwin.action_handler_t.__init__(self)
        self.hooks = hooks

    def activate(self, ctx):
        vu = ida_hexrays.get_widget_vdui(ctx.widget)
        if vu:
            loc = ida_moves.lochist_entry_t()
            if ida_kernwin.get_custom_viewer_location(loc, ctx.widget):
                coord = pseudo_line_t(
                    vu.cfunc.entry_ea,
                    _place_to_line_number(loc.place()))
                if coord in self.hooks.marked_lines.keys():
                    del self.hooks.marked_lines[coord]
                else:
                    self.hooks.marked_lines[coord] = self.COLOR_KEY
                ida_kernwin.refresh_custom_viewer(ctx.widget)

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_FOR_WIDGET \
            if ctx.widget_type == ida_kernwin.BWN_PSEUDOCODE \
            else ida_kernwin.AST_DISABLE_FOR_WIDGET


hooks = pseudocode_lines_rendering_hooks_t()
act_name = "example:colorize_pseudocode_line"
act_shortcut = "M"
if ida_kernwin.register_action(ida_kernwin.action_desc_t(
        act_name,
        "Mark pseudocode line",
        toggle_line_marked_ah_t(hooks),
        act_shortcut)):
    hooks.hook()
    print("Action registered. Please press '%s' in a pseudocode window to mark a line" % act_shortcut)



import binascii
import idaapi
import idautils

from ..updater.update import check_for_updates
from ..utils.cache_dumper import DumpAsker
from ..utils.common import *
from yaraforge.maker.dump_maker import *
from yaraforge.maker.instruction_maker import InstructionMaker
from ..utils.logger import *
from ..utils.logger import get_global_logger
from yaraforge.maker.yara_maker import YaraMaker

logger = get_global_logger(pathnames['logger_dir'])


class YaraForgePlugin(idaapi.plugin_t):
    flags = idaapi.PLUGIN_UNL
    comment = "YaraForge Plugin for IDA Pro Integrated with CAPA and mkYARA."
    help = "YaraForge Plugin"
    wanted_name = "YaraForge"
    wanted_hotkey = "Alt-Y"

    def init(self):
        try:
            initialize_logger()
            logger.info("YaraForge plugin initialized")
            print(f"{metadata['plugin_name']} plugin initialized")
            check_for_updates()
        except Exception as e:
            print(f"Error: Exception occurred during logger initialization, reason: {e}")
        return idaapi.PLUGIN_OK

    def term(self):
        try:
            logger.info("YaraForge plugin terminated")
            print(f"{metadata['plugin_name']} plugin terminated")
        except Exception as e:
            print(f"Error: Exception occurred during plugin termination, reason: {e}")

    def run(self, arg):
        try:
            logger.info("YaraForge plugin run started")

            file_hex_md5 = binascii.hexlify(idautils.GetInputFileMD5()).decode('ascii').lower()

            # Explore Netnodes
            explore_netnodes()

            # Make Directories
            make_dirs(pathnames.values())

            # Pretty Dump
            dump = DumpMaker(file_hex_md5)
            dump.make_pretty_dump()

            # Instruction Data
            inst = InstructionMaker(file_hex_md5)
            inst.make_instructions()

            # Yara Rule Generation
            yara = YaraMaker(file_hex_md5)
            yara.generate_rule()

            asker = DumpAsker()
            asker.ask_user_for_dump()



        except Exception as e:
            logger.error(f"Error occurred during YaraForge plugin execution: {e}")
            print(f"Error occurred during YaraForge plugin execution: {e}")


def PLUGIN_ENTRY():
    return YaraForgePlugin()



"""
summary: retrieve the strings that are present in the IDB

description:
  This uses `idautils.Strings` to iterate over the string literals
  that are present in the IDB. Contrary to @show_selected_strings,
  this will not require that the "Strings" window is opened & available.

see_also: show_selected_strings
"""

import ida_nalt
import idautils

s = idautils.Strings(False)
s.setup(strtypes=[ida_nalt.STRTYPE_C, ida_nalt.STRTYPE_C_16])
for i, v in enumerate(s):
    if v is None:
        print("Failed to retrieve string index %d" % i)
    else:
        print("%x: len=%d type=0x%x index=%d-> '%s'" % (v.ea, v.length, v.strtype, i, str(v)))



import json
from pathlib import Path

import capa

from yaraforge.metadata import pathnames
from yaraforge.utils.common import custom_json_serializer
from yaraforge.utils.logger import get_global_logger

logger = get_global_logger(pathnames['logger_dir'])


class DumpMaker:
    def __init__(self, file_hex_md5):
        """
        Initialize the DumpMaker object.
        :param file_hex_md5: The MD5 hash of the file.
        :return: None
        """
        self.file_hex_md5 = file_hex_md5
        self.logger = get_global_logger(pathnames['logger_dir'])

    def make_pretty_dump(self):
        """
        Make a pretty dump of the capa results.

        :return: None
        """
        self.logger.info(f"Starting pretty dump for {self.file_hex_md5}.")
        try:
            result_document = capa.ida.helpers.load_and_verify_cached_results()
            if result_document:
                json_data = json.dumps(result_document.model_dump(), default=custom_json_serializer, indent=4,
                                       ensure_ascii=False)

                pretty_dump = Path(pathnames['pretty_dump_dir']) / f"{self.file_hex_md5}_pretty_dump.json"
                with pretty_dump.open('w', encoding='utf-8') as file:
                    file.write(json_data)
                self.logger.info(f"PrettyDump has been saved to {pretty_dump}")
            else:
                self.logger.error("Failed to find or load CAPA results.")
        except Exception as e:
            self.logger.error(f"Error during pretty dump: {e}")
        return self





"""
summary: enumerate problems

description:
  Using the API to list all problem[atic situation]s that IDA
  encountered during analysis.
"""

import ida_ida
import ida_idaapi
import ida_problems

for ptype in [
        ida_problems.PR_NOBASE,
        ida_problems.PR_NONAME,
        ida_problems.PR_NOFOP,
        ida_problems.PR_NOCMT,
        ida_problems.PR_NOXREFS,
        ida_problems.PR_JUMP,
        ida_problems.PR_DISASM,
        ida_problems.PR_HEAD,
        ida_problems.PR_ILLADDR,
        ida_problems.PR_MANYLINES,
        ida_problems.PR_BADSTACK,
        ida_problems.PR_ATTN,
        ida_problems.PR_FINAL,
        ida_problems.PR_ROLLED,
        ida_problems.PR_COLLISION,
        ida_problems.PR_DECIMP,
]:
    plistdesc = ida_problems.get_problem_name(ptype)
    ea = ida_ida.inf_get_min_ea()
    while True:
        ea = ida_problems.get_problem(ptype, ea+1)
        if ea == ida_idaapi.BADADDR:
            break
        print("0x%08x: %s" % (ea, plistdesc))



from __future__ import print_function
#---------------------------------------------------------------------
# IDAPython - Python plugin for Interactive Disassembler
#
# (c) The IDAPython Team <idapython@googlegroups.com>
#
# All rights reserved.
#
# For detailed copyright information see the file COPYING in
# the root of the distribution archive.
#---------------------------------------------------------------------
#
# dex.py - module to access DEX-file related information
#
#---------------------------------------------------------------------
# pylint: disable=C0103, C0111, C0301, C0326, W0511, R0903
import sys
import ctypes
import idaapi
import ida_idaapi
import ida_bytes
import idc

uint8  = ctypes.c_ubyte
char   = ctypes.c_char
uint32 = ctypes.c_uint
uint64 = ctypes.c_uint64
uint16 = ctypes.c_ushort
ushort = uint16
# __EA64__ is set if IDA is running in 64-bit mode
__EA64__ = ida_idaapi.BADADDR == 0xFFFFFFFFFFFFFFFF
ea_t = uint64 if __EA64__ else uint32

# Dalvik indices are stored as uint32
def to_uint32(v):
    return int.from_bytes(v, byteorder='little') & 0xFFFFFFFF

# parse a ctypes struct from byte data in str_ at 'off'
def get_struct(str_, off, struct):
    s = struct()
    slen = ctypes.sizeof(s)
    bytebuf = str_[off:off+slen]
    fit = min(len(bytebuf), slen)
    if fit < slen:
        raise Exception("can't read struct: %d bytes available but %d required" % (fit, slen))
    ctypes.memmove(ctypes.addressof(s), bytebuf, fit)
    return s

_byte = ord if sys.version_info.major < 3 else lambda t: t

# unpack base address
def unpack_db(buf, off):
    x = 0
    if off < len(buf):
        x = _byte(buf[off])
        off += 1
    return (x, off)

def get_dw(buf, off):
    x = 0
    if off < len(buf):
        x = _byte(buf[off]) << 8
        off += 1
    if off < len(buf):
        x |= _byte(buf[off])
        off += 1
    return (x, off)

def unpack_dw(buf, off):
    (x, off) = unpack_db(buf, off)
    if (x & 0x80) == 0x80:
        if (x & 0xC0) == 0xC0:
            (x, off) = get_dw(buf, off)
        else:
            if off < len(buf):
                x = ((x & ~0x80) << 8) | _byte(buf[off])
                off += 1
    return (x, off)

def unpack_dd(buf, off):
    (x, off) = unpack_db(buf, off)
    if (x & 0x80) == 0x80:
        if (x & 0xC0) == 0xC0:
            if (x & 0xE0) == 0xE0:
                (xh, off) = get_dw(buf, off)
            else:
                xh = 0
                if off < len(buf):
                    xh = ((x & ~0xC0) << 8) | _byte(buf[off])
                    off += 1
            (xl, off) = get_dw(buf, off)
            x = (xh << 16) | xl
        else:
            if off < len(buf):
                x = ((x & ~0x80) << 8) | _byte(buf[off])
                off += 1
    return (x, off)

def unpack_dq(buf, off):
    (xl, off) = unpack_dd(buf, off)
    (xh, off) = unpack_dd(buf, off)
    x = (xh << 32) | xl
    if x > 0x8000000000000000:
        x = x - 0x10000000000000000
    return (x, off)

def unpack_ea(buf, off):
    if __EA64__:
        return unpack_dq(buf, off)
    else:
        return unpack_dd(buf, off)

def unpack_eavec(buf, base_ea):
    (n, off) = unpack_dw(buf, 0)
    ba = []
    old_ea = base_ea
    for i in range(0, n):
        (ea, off) = unpack_ea(buf, off)
        old_ea += ea
        ba.append(old_ea)
    return ba

#---------------------------------------------------------------------------
# This structure is used both for imported methods and locally defined ones
#
class dex_method(ctypes.LittleEndianStructure):
    # flags
    IS_LOCAL = 1
    HAS_CODE = 2
    _fields_ = [
        ("flags",          uint32), # Class type where this method is defined
        ("defaddr",          ea_t), # Address in file where the "definiton" (DexMethodId) is stored
        ("cname",          uint32), # Class type where this method is defined
        ("id",             uint32), # Id of method; key to look up name
        ("proto_ret",      uint32), # Name of return type
        ("proto_shorty",   uint32), # 'shorty' parameter descirptor name
        ("nparams",        ushort), # No of parameters to method. May be >32
        ("proto_params",uint32*32), # Name of types for the first 32 parameters
        ("access_flags",   uint32), # Access flags
        ("startAddr",       ea_t),  # Function start and end address
        ("endAddr",         ea_t),  #
        ("reg_total",      ushort), # Registers total, parameters and out
        ("reg_params",     ushort), #
        ("reg_out",        ushort), #
        ("catchHData",       ea_t), # offset to methods catch handler data
    ]
    def is_local(self):
        return (self.flags & dex_method.IS_LOCAL) != 0


"""
struct dex_field
{
  uint32 ctype, name, type;
  ea_t maddr;   // Address used for xrefs.
};

"""
class dex_field(ctypes.LittleEndianStructure):
    # flags
    _fields_ = [
        ("ctype",   uint32), #
        ("name",   uint32), #
        ("type",   uint32), #
        ("maddr",   ea_t), # Address used for xrefs.
    ]

"""
struct longname_director_t
{
  char zero;
  netnode node;
};
"""
class longname_director_t(ctypes.LittleEndianStructure):
    # flags
    _pack_ = 1
    _fields_ = [
        ("zero",   uint8), #
        ("node",   ea_t), # netnode index with the actual string blob
    ]


class Dex(object):

    # meta-data
    HASHVAL_MAGIC      = "version"      # Interface version
    HASHVAL_OPTIMIZED  = "optimized"    # 1 for optimized dex files, 0 - for others
    HASHVAL_DEXVERSION = "dex_version"  # DEX File version
    META_BASEADDRS     = 1              # eavec_t blob at index 0

    # ea-based indexes
    DEXCMN_STRING_ID  = ord('S')    # string ea => string_id
    DEXCMN_METHOD_ID  = ord('M')    # dex_method::func.start_ea => method_id
    DEXCMN_TRY_TYPES  = ord('E')    # ea (handler start) => list of type_id, handled types
    DEXCMN_TRY_IDS    = ord('Y')    # ea (handler start) => list of try_item_id
    DEXCMN_DEBINFO    = ord('D')    # line start ea => dex_lineinfo_t
    DEXCMN_DEBSTR     = ord('B')    # line start ea => human readable debug info string

    # var indexes
    DEXVAR_STRING_IDS = ord('S')    # string_id => ea
    DEXVAR_TYPE_IDS   = ord('T')    # type_id => descriptor_idx
    DEXVAR_TYPE_STR   = ord('U')    # type_id => type string (possible user redefined), char data
    DEXVAR_TYPE_STRO  = ord('V')    # type_id => type string (original), char data
    DEXVAR_METHOD     = ord('M')    # method_id => struct dex_method, supval
    DEXVAR_METH_STR   = ord('N')    # method_id => method name, char data
    DEXVAR_METH_STRO  = ord('O')    # method_id => method name fromdex file, char data
    DEXVAR_FIELD      = ord('F')    # field_id => struct dex_field
    DEXVAR_TRYLIST    = ord('Y')    # method_id => try_item

    # debug info representation
    DEBINFO_LINEINFO = 1        # Line start EA => dex_lineinfo_t

    #---------------------------------------------------------------------------
    def __init__(self):
        self.nn_meta = idaapi.netnode("$ dex_meta")
        self.nn_cmn = idaapi.netnode("$ dex_cmn")
        packed = self.nn_meta.getblob(0, Dex.META_BASEADDRS)
        self.baseaddrs = unpack_eavec(packed, 0)
        self.nn_vars = []
        self.nn_vars.append(idaapi.netnode("$ dex_var"))
        for i in range(2, len(self.baseaddrs) + 1):
            nn_var_name = "$ dex_var%d" % i
            self.nn_vars.append(idaapi.netnode(nn_var_name))

    #---------------------------------------------------------------------------
    def get_dexnum(self, from_ea):
        dexnum = 0
        for ba in self.baseaddrs:
            if from_ea < ba:
                break
            dexnum += 1
        return dexnum

    #---------------------------------------------------------------------------
    def get_nn_var(self, from_ea):
        return self.nn_vars[self.get_dexnum(from_ea) - 1]

    #---------------------------------------------------------------------------
    ACCESS_FLAGS = {
        "public"        : 0x00000001,
        "private"       : 0x00000002,
        "protected"     : 0x00000004,
        "static"        : 0x00000008,
        "final"         : 0x00000010,
        "synchronized"  : 0x00000020,
        "volatile"      : 0x00000040,
        "bridge"        : 0x00000040,
        "transient"     : 0x00000080,
        "varargs"       : 0x00000080,
        "native"        : 0x00000100,
        "interface"     : 0x00000200,
        "abstract"      : 0x00000400,
        "strictfp"      : 0x00000800,
        "synthetic"     : 0x00001000,
        "annotation"    : 0x00002000,
        "enum"          : 0x00004000,
        "constructor"   : 0x00010000,
        "dsynchronized" : 0x00020000, }

    #---------------------------------------------------------------------------
    @staticmethod
    def access_string(flags):
        res = ""
        for access_bit in ("synchronized", "synthetic", "public",
                           "private", "protected", "interface",
                           "abstract", "strictfp", "final",
                           "native", "static"):
            if flags & Dex.ACCESS_FLAGS[access_bit] != 0:
                res += " " + access_bit
        return res[1:] if res else ""

    #---------------------------------------------------------------------------
    @staticmethod
    def as_string(s):
        return s.decode("UTF-8") if sys.version_info.major >= 3 else s

    #---------------------------------------------------------------------------
    def idx_to_ea(self, from_ea, idx, tag):
        nn_var = self.get_nn_var(from_ea)
        return nn_var.eaget_idx(idx, tag)

    #---------------------------------------------------------------------------
    def get_string(self, from_ea, string_idx):
        addr = self.idx_to_ea(from_ea, string_idx, Dex.DEXVAR_STRING_IDS)
        if addr == ida_idaapi.BADADDR:
            return None
        length = ida_bytes.get_max_strlit_length(addr, idc.STRTYPE_C, ida_bytes.ALOPT_IGNHEADS|ida_bytes.ALOPT_IGNPRINT)
        raw = ida_bytes.get_strlit_contents(addr, length, idc.STRTYPE_C)
        return Dex.as_string(raw)

    def get_method_idx(self, ea):
        return to_uint32(self.nn_cmn.supval_ea(ea, Dex.DEXCMN_METHOD_ID))

    def get_method(self, from_ea, method_idx):
        nn_var = self.get_nn_var(from_ea)
        val = nn_var.supval(method_idx, Dex.DEXVAR_METHOD)
        if len(val) != ctypes.sizeof(dex_method):
            print("bad data in DEXVAR_METHOD for index 0x%X" % method_idx)
            return None
        method = get_struct(val,0, dex_method)
        return method

    #---------------------------------------------------------------------------
    @staticmethod
    def get_string_by_index(node, idx, tag):
        if idx is None:
            return None
        val = node.supval(idx, tag)
        # check for long line
        if len(val) == ctypes.sizeof(longname_director_t):
            longname_director = get_struct(val, 0, longname_director_t)
            if longname_director.zero == 0:
                nn = idaapi.netnode(longname_director.node)
                return Dex.as_string(nn.getblob(0, tag)[:-1])
        if len(val) > 0:
            return Dex.as_string(val[:-1])
        return ""

    #---------------------------------------------------------------------------
    # Converts a single-char primitive type into its human-readable equivalent
    PRIMITVE_TYPES = {
        'B': "byte",
        'C': "char",
        'D': "double",
        'F': "float",
        'I': "int",
        'J': "long",
        'S': "short",
        'V': "void",
        'Z': "boolean",
        'L': "ref" }
    @staticmethod
    def _primitive_type_label(typechar):
        if typechar in Dex.PRIMITVE_TYPES:
            return Dex.PRIMITVE_TYPES[typechar]
        return "UNKNOWN"

    @staticmethod
    def is_wide_type(typechar):
        return typechar[0] == 'J' or typechar[0] == 'D'

    #---------------------------------------------------------------------------
    # Converts a type descriptor to human-readable "dotted" form.  For
    # example, "Ljava/lang/String;" becomes "java.lang.String", and
    # "[I" becomes "int[]".  Also converts '$' to '.', which means this
    # form can't be converted back to a descriptor.
    @staticmethod
    def decorate_java_typename(desc):
        target_len = len(desc)
        offset = 0
        # strip leading [s; will be added to end
        while target_len > 1 and desc[offset] == '[':
            offset += 1
            target_len -= 1
        array_depth = offset
        if target_len == 1:
            # primitive type
            desc = Dex._primitive_type_label(desc[offset])
            offset = 0
            target_len = len(desc)
        else:
            # account for leading 'L' and trailing ';'
            if target_len >= 2 and desc[offset] == 'L' and desc[offset + target_len - 1] == ';':
                target_len -= 2
                offset += 1
        # copy class name over
        res = ""
        for _i in range(0, target_len):
            ch = desc[offset + _i]
            res += '.' if ch == '/' else ch
        # add the appropriate number of brackets for arrays
        res += "[]"*array_depth
        return res

    #---------------------------------------------------------------------------
    def get_type_string(self, from_ea, type_idx):
        nn_var = self.get_nn_var(from_ea)
        return Dex.get_string_by_index(nn_var, type_idx, Dex.DEXVAR_TYPE_STR)

    def get_method_name(self, from_ea, method_idx):
        nn_var = self.get_nn_var(from_ea)
        return Dex.get_string_by_index(nn_var, method_idx, Dex.DEXVAR_METH_STR)

    def get_parameter_name(self, from_ea, idx):
        return self.get_string(from_ea, idx)

    #---------------------------------------------------------------------------
    @staticmethod
    def get_short_type_name(longname):
        if not longname:
            return "unknown"
        deco = Dex.decorate_java_typename(longname)
        if not deco:
            return "unknown"
        start = deco.rfind('.')
        if start == -1:
            start = 0
        else:
            start += 1
        return deco[start:].replace('<', '_').replace('>', '_')

    @staticmethod
    def get_full_type_name(longname):
        if not longname:
            return "unknown"
        return Dex.decorate_java_typename(longname)

    #---------------------------------------------------------------------------
    def get_short_method_name(self, method):
        res = Dex.get_short_type_name(self.get_type_string(method.defaddr, method.cname))
        res += '.'
        res += self.get_method_name(method.defaddr, method.id)
        res += '@'
        res += self.get_string(method.defaddr, method.proto_shorty)
        return res

    def get_full_method_name(self, method):
        res = Dex.get_full_type_name(self.get_type_string(method.defaddr, method.proto_ret))
        res += ' '
        res += self.get_full_type_name(self.get_type_string(method.defaddr, method.cname))
        res += '.'
        res += self.get_method_name(method.defaddr, method.id)

    def get_call_method_name(self, method):
        shorty = self.get_string(method.defaddr, method.proto_shorty)
        res = Dex._primitive_type_label(shorty[0])
        res += ' '
        res += Dex.get_short_type_name(self.get_type_string(method.defaddr, method.cname))
        res += '.'
        res += self.get_method_name(method.defaddr, method.id)
        res += '('
        last_idx = len(shorty) - 1
        for s in range(1, last_idx + 1):
            res += Dex._primitive_type_label(shorty[s])
            if s != last_idx:
                res += ", "
        res += ')'
        return res


    #---------------------------------------------------------------------------
    def get_field(self, from_ea, field_idx):
        nn_var = self.get_nn_var(from_ea)
        val = nn_var.supval(field_idx, Dex.DEXVAR_FIELD)
        if len(val) != ctypes.sizeof(dex_field):
            print("bad data in DEXVAR_FIELD for index 0x%X" % field_idx)
            return None
        field = get_struct(val,0, dex_field)
        return field


    def get_field_name(self, from_ea, field_idx):
        field = self.get_field(from_ea, field_idx)
        return self.get_string(from_ea, field.name)


    def get_full_field_name(self, field_idx, field, field_name):
        res = Dex.get_full_type_name(self.get_type_string(field.maddr, field.type))
        res += ' '
        res += Dex.get_full_type_name(self.get_type_string(field.maddr, field_idx))
        res += '.'
        res += field_name if field_name else self.get_field_name(field.maddr, field_idx)
        return res


    def get_short_field_name(self, field_idx, field, field_name):
        res = Dex.get_short_type_name(self.get_type_string(field.maddr, field.ctype))
        res += '_'
        res += field_name if field_name else self.get_field_name(field.maddr, field_idx)


#---------------------------------------------------------------------------
if __name__ == '__main__':
    dex = Dex()
    # reproduce IDA function header
    f = idaapi.get_func(here())
    if not f:
        print("ERROR: must be in a function!")
        exit(1)

    func_start_ea = f.start_ea
    methno = dex.get_method_idx(func_start_ea)
    func_method = dex.get_method(func_start_ea, methno)
    if func_method is None:
        print("ERROR: Missing method info")
        exit(1)
    out = ""
    # Return type
    out += Dex.access_string(func_method.access_flags) + " "
    method_proto = dex.get_type_string(func_start_ea, func_method.proto_ret)
    if method_proto:
        out += Dex.get_full_type_name(method_proto)
    else:
        out += "%x" % func_method.proto_ret
    out += ' '
    # Class name
    method_classnm = dex.get_type_string(func_start_ea, func_method.cname)
    if method_classnm:
        out += Dex.get_full_type_name(method_classnm)
    else:
        out += "%x" % func_method.cname
    out += '.'
    # Method name
    method_name = dex.get_method_name(func_start_ea, methno)
    if method_name:
        out += method_name
    else:
        out += "%x" % methno
    # Method parameters
    if func_method.nparams == 0:
        print(out + "()")
    else:
        print(out + "(")
        out = ""
        maxp = min(func_method.nparams, 32)
        start_reg = func_method.reg_total - func_method.reg_params
        if func_method.access_flags & Dex.ACCESS_FLAGS["static"] == 0:
            start_reg += 1
        for i in range(0, maxp):
            ptype = dex.get_type_string(func_start_ea, func_method.proto_params[i])

            out = "  %s " % dex.get_full_type_name(ptype)
            regbuf = "v%u" % start_reg
            start_reg += 1
            r = idaapi.find_regvar(f, f.start_ea, regbuf)
            if r is None:
                out += regbuf
                if Dex.is_wide_type(ptype):
                    out += ':'
                    regbuf = "v%u" % start_reg
                    start_reg += 1
            else:
                out += r.user
            out += ')' if i + 1 == maxp else ','
            print(out)



"""
summary: list all functions (and xrefs) in segment

description:
  List all the functions in the current segment, as well as
  all the cross-references to them.

keywords: xrefs

see_also: list_segment_functions_using_idautils
"""

#
# Reference Lister
#
# List all functions and all references to them in the current section.
#
# Implemented using direct IDA Plugin API calls
#

import ida_kernwin
import ida_segment
import ida_funcs
import ida_xref
import ida_idaapi

def main():
    # Get current ea
    ea = ida_kernwin.get_screen_ea()

    # Get segment class
    seg = ida_segment.getseg(ea)

    # Loop from segment start to end
    func_ea = seg.start_ea

    # Get a function at the start of the segment (if any)
    func = ida_funcs.get_func(func_ea)
    if func is None:
        # No function there, try to get the next one
        func = ida_funcs.get_next_func(func_ea)

    seg_end = seg.end_ea
    while func is not None and func.start_ea < seg_end:
        funcea = func.start_ea
        print("Function %s at 0x%x" % (ida_funcs.get_func_name(funcea), funcea))

        xb = ida_xref.xrefblk_t()
        for ref in xb.crefs_to(funcea):
            print("  called from %s(0x%x)" % (ida_funcs.get_func_name(ref), ref))

        func = ida_funcs.get_next_func(funcea)


main()



"""
summary: a custom microcode block optimization rule (resolve `goto` chains)

description:
  Installs a custom microcode block optimization rule,
  to transform:

        goto L1
        ...
      L1:
        goto L2

  into

        goto L2

  In other words we fix a goto target if it points to a chain of gotos.
  This improves the decompiler output in some cases.
"""

import ida_bytes
import ida_range
import ida_kernwin
import ida_hexrays
import ida_typeinf
import ida_idaapi

class goto_optimizer_t(ida_hexrays.optblock_t):
    def func(self, blk):
        if self.handle_goto_chain(blk):
            return 1
        return 0

    def handle_goto_chain(self, blk):
        mgoto = blk.tail
        if not mgoto or mgoto.opcode != ida_hexrays.m_goto:
            return False

        visited = []
        t0 = mgoto.l.b
        i = t0
        mba = blk.mba

        # follow the goto chain
        while True:
            if i in visited:
                return False
            visited.append(i)
            b = mba.get_mblock(i)
            m2 = ida_hexrays.getf_reginsn(b.head)
            if not m2 or m2.opcode != ida_hexrays.m_goto:
                break
            i = m2.l.b

        if i == t0:
            return False # not a chain

        # all ok, found a goto chain
        mgoto.l.b = i # jump directly to the end of the chain

        # fix the successor/predecessor lists
        blk.succset[0] = i
        mba.get_mblock(i).predset.add(blk.serial)
        mba.get_mblock(t0).predset._del(blk.serial)

        # since we changed the control flow graph, invalidate the use/def chains.
        # stricly speaking it is not really necessary in our plugin because
        # we did not move around any microcode operands.
        mba.mark_chains_dirty()

        # it is a good idea to verify microcode after each change
        # however, it may be time consuming, so comment it out eventually
        mba.verify(True);
        return True

# --------------------------------------------------------------------------
# a plugin interface, boilerplate code
class my_plugin_t(ida_idaapi.plugin_t):
    flags = ida_idaapi.PLUGIN_HIDE
    wanted_name = "Optimize goto chains (IDAPython)"
    wanted_hotkey = ""
    comment = "Sample plugin11 for Hex-Rays decompiler"
    help = ""
    def init(self):
        if ida_hexrays.init_hexrays_plugin():
            self.optimizer = goto_optimizer_t()
            self.optimizer.install()
            return ida_idaapi.PLUGIN_KEEP # keep us in the memory
    def term(self):
        self.optimizer.remove()
    def run(self, arg):
        if arg == 1:
            return self.optimizer.remove()
        elif arg == 2:
            return self.optimizer.install()

def PLUGIN_ENTRY():
    return my_plugin_t()




"""
Functions that provide the lowest level public interface to the database.
Namely, we use Btree. To learn more about BTree:

\link{https://en.wikipedia.org/wiki/B-tree}

We do not use Btree directly. Instead, we have another layer built on the top of
Btree. Here is a brief explanation of this layer.

An object called "netnode" is modeled on the top of Btree. Each netnode has a
unique id: a 32-bit value (64-bit for ida64). Initially there is a trivial
mapping of the linear addresses used in the program to netnodes (later this
mapping may be modified using ea2node and node2ea functions; this is used for
fast database rebasings). If we have additional information about an address
(for example, a comment is attached to it), this information is stored in the
corresponding netnode. See nalt.hpp to see how the kernel uses netnodes. Also,
some netnodes have no corresponding linear address (however, they still have an
id). They are used to store information not related to a particular address.

Each netnode _may_ have the following attributes:

* a name: an arbitrary non-empty string, up to 255KB-1 bytes
* a value: arbitrary sized object, max size is MAXSPECSIZE
* altvals: a sparse array of 32-bit values. indexes in this array may be 8-bit
or 32-bit values
* supvals: an array of arbitrary sized objects. (size of each object is limited
by MAXSPECSIZE) indexes in this array may be 8-bit or 32-bit values
* charvals: a sparse array of 8-bit values. indexes in this array may be 8-bit
or 32-bit values
* hashvals: a hash (an associative array). indexes in this array are strings
values are arbitrary sized (max size is MAXSPECSIZE)

Initially a new netnode contains no information at all so no disk space is used
for it. As you add new information, the netnode grows.

All arrays that are attached to the netnode behave in the same manner.
Initially:
* all members of altvals/charvals array are zeroes
* all members of supvals/hashvals array are undefined

If you need to store objects bigger that MAXSPECSIZE, please note that there are
high-level functions to store arbitrary sized objects in supvals. See
setblob/getblob and other blob-related functions.

You may use netnodes to store additional information about the program.
Limitations on the use of netnodes are the following:

* use netnodes only if you could not find a kernel service to store your type of
information
* do not create netnodes with valid identifier names. Use the "$ " prefix (or
any other prefix with characters not allowed in the identifiers for the names of
your netnodes. Although you will probably not destroy anything by accident,
using already defined names for the names of your netnodes is still discouraged.
* you may create as many netnodes as you want (creation of an unnamed netnode
does not increase the size of the database). however, since each netnode has a
number, creating too many netnodes could lead to the exhaustion of the netnode
numbers (the numbering starts at 0xFF000000)
* remember that netnodes are automatically saved to the disk by the kernel.

Advanced info:

In fact a netnode may contain up to 256 arrays of arbitrary sized objects (not
only the 4 listed above). Each array has an 8-bit tag. Usually tags are
represented by character constants. For example, altvals and supvals are simply
2 of 256 arrays, with the tags 'A' and 'S' respectively."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_netnode
else:
    import _ida_netnode

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_netnode.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

BADNODE = _ida_netnode.BADNODE
r"""
A number to represent a bad netnode reference.
"""

SIZEOF_nodeidx_t = _ida_netnode.SIZEOF_nodeidx_t

class netnode(object):
    r"""
    Proxy of C++ netnode class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self, num=nodeidx_t(-1)) -> netnode

        @param num: nodeidx_t

        __init__(self, _name, namlen=0, do_create=False) -> netnode

        @param _name: char const *
        @param namlen: size_t
        @param do_create: bool
        """
        _ida_netnode.netnode_swiginit(self, _ida_netnode.new_netnode(*args))

    @staticmethod
    def exist(*args) -> "bool":
        r"""
        exist(_name) -> bool
        Does the netnode with the specified name exist?

        @param _name: (C++: const char *) char const *
        """
        return _ida_netnode.netnode_exist(*args)

    def create(self, *args) -> "bool":
        r"""
        create(self, _name, namlen=0) -> bool
        Create unnamed netnode.
        @retval 1: ok
        @retval 0: should not happen, indicates internal error

        @param _name: char const *
        @param namlen: size_t

        create(self) -> bool
        """
        return _ida_netnode.netnode_create(self, *args)

    def kill(self, *args) -> "void":
        r"""
        kill(self)
        Delete a netnode with all information attached to it.
        """
        return _ida_netnode.netnode_kill(self, *args)

    def get_name(self, *args) -> "ssize_t":
        r"""
        get_name(self) -> ssize_t
        Get the netnode name.

        @return: -1 if netnode is unnamed (buf is untouched in this case), otherwise the
                 name length
        """
        return _ida_netnode.netnode_get_name(self, *args)

    def rename(self, *args) -> "bool":
        r"""
        rename(self, newname, namlen=0) -> bool
        Rename a netnode.

        @param newname: (C++: const char *) new name of netnode. nullptr or "" means to delete name. names
                        of user-defined netnodes must have the "$ " prefix in order to
                        avoid clashes with program byte names.
        @param namlen: (C++: size_t) length of new name. if not specified, it will be calculated using
                       strlen()
        @retval 1: ok
        @retval 0: failed, newname is already used
        """
        return _ida_netnode.netnode_rename(self, *args)

    def valobj(self, *args) -> "ssize_t":
        r"""
        valobj(self) -> ssize_t
        Get value of netnode. Netnode values are arbitrary sized objects with max size
        is MAXSPECSIZE. NB: do not use this function for strings - see valstr().

        @return: length of value, -1 if no value present
        """
        return _ida_netnode.netnode_valobj(self, *args)

    def valstr(self, *args) -> "ssize_t":
        r"""
        valstr(self) -> ssize_t
        @see: valstr(qstring *buf) const
        """
        return _ida_netnode.netnode_valstr(self, *args)

    def set(self, *args) -> "bool":
        r"""
        set(self, value) -> bool
        Set value of netnode.

        @param value: (C++: const void *) pointer to value
        @return: 1 - ok
        """
        return _ida_netnode.netnode_set(self, *args)

    def delvalue(self, *args) -> "bool":
        r"""
        delvalue(self) -> bool
        Delete value of netnode.
        @retval 1: ok
        @retval 0: failed, netnode is bad or other error
        """
        return _ida_netnode.netnode_delvalue(self, *args)

    def set_long(self, *args) -> "bool":
        r"""
        set_long(self, x) -> bool
        Value of netnode as a long number:

        @param x: (C++: nodeidx_t)
        """
        return _ida_netnode.netnode_set_long(self, *args)

    def value_exists(self, *args) -> "bool":
        r"""
        value_exists(self) -> bool
        """
        return _ida_netnode.netnode_value_exists(self, *args)

    def long_value(self, *args) -> "nodeidx_t":
        r"""
        long_value(self) -> nodeidx_t
        """
        return _ida_netnode.netnode_long_value(self, *args)

    def altval(self, *args) -> "nodeidx_t":
        r"""
        altval(self, alt, tag=atag) -> nodeidx_t
        Get altval element of the specified array.

        @param alt: (C++: nodeidx_t) index into array of altvals
        @param tag: (C++: uchar) tag of array. may be omitted
        @return: value of altval element. nonexistent altval members are returned as
                 zeroes
        """
        return _ida_netnode.netnode_altval(self, *args)

    def altval_ea(self, *args) -> "nodeidx_t":
        r"""
        altval_ea(self, ea, tag=atag) -> nodeidx_t

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_altval_ea(self, *args)

    def altset(self, *args) -> "bool":
        r"""
        altset(self, alt, value, tag=atag) -> bool
        Set value of altval array.

        @param alt: (C++: nodeidx_t) index into array of altvals
        @param value: (C++: nodeidx_t) new value of altval element
        @param tag: (C++: uchar) tag of array
        @retval 1: ok
        @retval 0: failed, normally should not occur
        """
        return _ida_netnode.netnode_altset(self, *args)

    def altset_ea(self, *args) -> "bool":
        r"""
        altset_ea(self, ea, value, tag=atag) -> bool

        @param ea: ea_t
        @param value: nodeidx_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_altset_ea(self, *args)

    def altdel_ea(self, *args) -> "bool":
        r"""
        altdel_ea(self, ea, tag=atag) -> bool

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_altdel_ea(self, *args)

    def easet(self, *args) -> "bool":
        r"""
        easet(self, ea, addr, tag) -> bool
        Store/retrieve/delete an address value in the netnode that corresponds to an
        address.

        @param ea: (C++: ea_t)
        @param addr: (C++: ea_t)
        @param tag: (C++: uchar)
        """
        return _ida_netnode.netnode_easet(self, *args)

    def eaget(self, *args) -> "ea_t":
        r"""
        eaget(self, ea, tag) -> ea_t

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_eaget(self, *args)

    def eadel(self, *args) -> "bool":
        r"""
        eadel(self, ea, tag) -> bool

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_eadel(self, *args)

    def easet_idx(self, *args) -> "bool":
        r"""
        easet_idx(self, idx, addr, tag) -> bool

        @param idx: nodeidx_t
        @param addr: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_easet_idx(self, *args)

    def eaget_idx(self, *args) -> "ea_t":
        r"""
        eaget_idx(self, idx, tag) -> ea_t

        @param idx: nodeidx_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_eaget_idx(self, *args)

    def easet_idx8(self, *args) -> "bool":
        r"""
        easet_idx8(self, idx, addr, tag) -> bool

        @param idx: uchar
        @param addr: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_easet_idx8(self, *args)

    def eaget_idx8(self, *args) -> "ea_t":
        r"""
        eaget_idx8(self, idx, tag) -> ea_t

        @param idx: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_eaget_idx8(self, *args)

    def eadel_idx8(self, *args) -> "bool":
        r"""
        eadel_idx8(self, idx, tag) -> bool

        @param idx: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_eadel_idx8(self, *args)

    def altfirst(self, *args) -> "nodeidx_t":
        r"""
        altfirst(self, tag=atag) -> nodeidx_t
        Get first existing element of altval array.

        @param tag: (C++: uchar) tag of array
        @return: index of first existing element of altval array, BADNODE if altval
                 array is empty
        """
        return _ida_netnode.netnode_altfirst(self, *args)

    def altnext(self, *args) -> "nodeidx_t":
        r"""
        altnext(self, cur, tag=atag) -> nodeidx_t
        Get next existing element of altval array.

        @param cur: (C++: nodeidx_t) current index
        @param tag: (C++: uchar) tag of array
        @return: index of the next existing element of altval array, BADNODE if no more
                 altval array elements exist
        """
        return _ida_netnode.netnode_altnext(self, *args)

    def altlast(self, *args) -> "nodeidx_t":
        r"""
        altlast(self, tag=atag) -> nodeidx_t
        Get last element of altval array.

        @param tag: (C++: uchar) tag of array
        @return: index of last existing element of altval array, BADNODE if altval array
                 is empty
        """
        return _ida_netnode.netnode_altlast(self, *args)

    def altprev(self, *args) -> "nodeidx_t":
        r"""
        altprev(self, cur, tag=atag) -> nodeidx_t
        Get previous existing element of altval array.

        @param cur: (C++: nodeidx_t) current index
        @param tag: (C++: uchar) tag of array
        @return: index of the previous existing element of altval array, BADNODE if no
                 more altval array elements exist
        """
        return _ida_netnode.netnode_altprev(self, *args)

    def altshift(self, *args) -> "size_t":
        r"""
        altshift(self, _from, to, size, tag=atag) -> size_t
        Shift the altval array elements. Moves the array elements at (from..from+size)
        to (to..to+size)

        @param from: (C++: nodeidx_t)
        @param to: (C++: nodeidx_t)
        @param size: (C++: nodeidx_t)
        @param tag: (C++: uchar)
        @return: number of shifted elements
        """
        return _ida_netnode.netnode_altshift(self, *args)

    def charval(self, *args) -> "uchar":
        r"""
        charval(self, alt, tag) -> uchar

        @param alt: nodeidx_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_charval(self, *args)

    def charset(self, *args) -> "bool":
        r"""
        charset(self, alt, val, tag) -> bool

        @param alt: nodeidx_t
        @param val: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_charset(self, *args)

    def chardel(self, *args) -> "bool":
        r"""
        chardel(self, alt, tag) -> bool

        @param alt: nodeidx_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_chardel(self, *args)

    def charval_ea(self, *args) -> "uchar":
        r"""
        charval_ea(self, ea, tag) -> uchar

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_charval_ea(self, *args)

    def charset_ea(self, *args) -> "bool":
        r"""
        charset_ea(self, ea, val, tag) -> bool

        @param ea: ea_t
        @param val: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_charset_ea(self, *args)

    def chardel_ea(self, *args) -> "bool":
        r"""
        chardel_ea(self, ea, tag) -> bool

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_chardel_ea(self, *args)

    def charfirst(self, *args) -> "nodeidx_t":
        r"""
        charfirst(self, tag) -> nodeidx_t

        @param tag: uchar
        """
        return _ida_netnode.netnode_charfirst(self, *args)

    def charnext(self, *args) -> "nodeidx_t":
        r"""
        charnext(self, cur, tag) -> nodeidx_t

        @param cur: nodeidx_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_charnext(self, *args)

    def charlast(self, *args) -> "nodeidx_t":
        r"""
        charlast(self, tag) -> nodeidx_t

        @param tag: uchar
        """
        return _ida_netnode.netnode_charlast(self, *args)

    def charprev(self, *args) -> "nodeidx_t":
        r"""
        charprev(self, cur, tag) -> nodeidx_t

        @param cur: nodeidx_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_charprev(self, *args)

    def charshift(self, *args) -> "size_t":
        r"""
        charshift(self, _from, to, size, tag) -> size_t

        @param from: nodeidx_t
        @param to: nodeidx_t
        @param size: nodeidx_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_charshift(self, *args)

    def altval_idx8(self, *args) -> "nodeidx_t":
        r"""
        altval_idx8(self, alt, tag) -> nodeidx_t

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_altval_idx8(self, *args)

    def altset_idx8(self, *args) -> "bool":
        r"""
        altset_idx8(self, alt, val, tag) -> bool

        @param alt: uchar
        @param val: nodeidx_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_altset_idx8(self, *args)

    def altdel_idx8(self, *args) -> "bool":
        r"""
        altdel_idx8(self, alt, tag) -> bool

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_altdel_idx8(self, *args)

    def altfirst_idx8(self, *args) -> "nodeidx_t":
        r"""
        altfirst_idx8(self, tag) -> nodeidx_t

        @param tag: uchar
        """
        return _ida_netnode.netnode_altfirst_idx8(self, *args)

    def altnext_idx8(self, *args) -> "nodeidx_t":
        r"""
        altnext_idx8(self, cur, tag) -> nodeidx_t

        @param cur: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_altnext_idx8(self, *args)

    def altlast_idx8(self, *args) -> "nodeidx_t":
        r"""
        altlast_idx8(self, tag) -> nodeidx_t

        @param tag: uchar
        """
        return _ida_netnode.netnode_altlast_idx8(self, *args)

    def altprev_idx8(self, *args) -> "nodeidx_t":
        r"""
        altprev_idx8(self, cur, tag) -> nodeidx_t

        @param cur: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_altprev_idx8(self, *args)

    def charval_idx8(self, *args) -> "uchar":
        r"""
        charval_idx8(self, alt, tag) -> uchar

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_charval_idx8(self, *args)

    def charset_idx8(self, *args) -> "bool":
        r"""
        charset_idx8(self, alt, val, tag) -> bool

        @param alt: uchar
        @param val: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_charset_idx8(self, *args)

    def chardel_idx8(self, *args) -> "bool":
        r"""
        chardel_idx8(self, alt, tag) -> bool

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_chardel_idx8(self, *args)

    def charfirst_idx8(self, *args) -> "nodeidx_t":
        r"""
        charfirst_idx8(self, tag) -> nodeidx_t

        @param tag: uchar
        """
        return _ida_netnode.netnode_charfirst_idx8(self, *args)

    def charnext_idx8(self, *args) -> "nodeidx_t":
        r"""
        charnext_idx8(self, cur, tag) -> nodeidx_t

        @param cur: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_charnext_idx8(self, *args)

    def charlast_idx8(self, *args) -> "nodeidx_t":
        r"""
        charlast_idx8(self, tag) -> nodeidx_t

        @param tag: uchar
        """
        return _ida_netnode.netnode_charlast_idx8(self, *args)

    def charprev_idx8(self, *args) -> "nodeidx_t":
        r"""
        charprev_idx8(self, cur, tag) -> nodeidx_t

        @param cur: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_charprev_idx8(self, *args)

    def altdel(self, *args) -> "bool":
        r"""
        altdel(self, alt, tag=atag) -> bool
        Delete all elements of altval array. This function may be applied to 32-bit and
        8-bit altval arrays. This function deletes the whole altval array.

        @param alt: nodeidx_t
        @param tag: uchar

        @return: success
        altdel(self) -> bool
        """
        return _ida_netnode.netnode_altdel(self, *args)

    def altdel_all(self, *args) -> "bool":
        r"""
        altdel_all(self, tag=atag) -> bool
        Delete all elements of the specified altval array. This function may be applied
        to 32-bit and 8-bit altval arrays. This function deletes the whole altval array.

        @param tag: (C++: uchar) tag of array
        @return: success
        """
        return _ida_netnode.netnode_altdel_all(self, *args)

    def supval(self, *args) -> "ssize_t":
        r"""
        supval(self, alt, tag=stag) -> ssize_t
        Get value of the specified supval array element. NB: do not use this function to
        retrieve strings, see supstr()!

        @param alt: (C++: nodeidx_t) index into array of supvals
        @param tag: (C++: uchar) tag of array. Default: stag
        @return: size of value, -1 if element doesn't exist
        """
        return _ida_netnode.netnode_supval(self, *args)

    def supval_ea(self, *args) -> "ssize_t":
        r"""
        supval_ea(self, ea, tag=stag) -> ssize_t

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_supval_ea(self, *args)

    def supstr(self, *args) -> "ssize_t":
        r"""
        supstr(self, alt, tag=stag) -> ssize_t
        @see: supstr(qstring *buf, nodeidx_t alt, uchar tag=stag) const

        @param alt: (C++: nodeidx_t)
        @param tag: (C++: uchar)
        """
        return _ida_netnode.netnode_supstr(self, *args)

    def supstr_ea(self, *args) -> "ssize_t":
        r"""
        supstr_ea(self, ea, tag=stag) -> ssize_t

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_supstr_ea(self, *args)

    def supdel_ea(self, *args) -> "bool":
        r"""
        supdel_ea(self, ea, tag=stag) -> bool

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_supdel_ea(self, *args)

    def lower_bound(self, *args) -> "nodeidx_t":
        r"""
        lower_bound(self, cur, tag=stag) -> nodeidx_t
        Get lower bound of existing elements of supval array.

        @param cur: (C++: nodeidx_t) current index
        @param tag: (C++: uchar) tag of array
        @return: index of first existing element of supval array >= cur BADNODE if
                 supval array is empty
        """
        return _ida_netnode.netnode_lower_bound(self, *args)

    def lower_bound_ea(self, *args) -> "nodeidx_t":
        r"""
        lower_bound_ea(self, ea, tag=stag) -> nodeidx_t

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_lower_bound_ea(self, *args)

    def supfirst(self, *args) -> "nodeidx_t":
        r"""
        supfirst(self, tag=stag) -> nodeidx_t
        Get first existing element of supval array.

        @param tag: (C++: uchar) tag of array
        @return: index of first existing element of supval array, BADNODE if supval
                 array is empty
        """
        return _ida_netnode.netnode_supfirst(self, *args)

    def supnext(self, *args) -> "nodeidx_t":
        r"""
        supnext(self, cur, tag=stag) -> nodeidx_t
        Get next existing element of supval array.

        @param cur: (C++: nodeidx_t) current index
        @param tag: (C++: uchar) tag of array
        @return: index of the next existing element of supval array, BADNODE if no more
                 supval array elements exist
        """
        return _ida_netnode.netnode_supnext(self, *args)

    def suplast(self, *args) -> "nodeidx_t":
        r"""
        suplast(self, tag=stag) -> nodeidx_t
        Get last existing element of supval array.

        @param tag: (C++: uchar) tag of array
        @return: index of last existing element of supval array, BADNODE if supval array
                 is empty
        """
        return _ida_netnode.netnode_suplast(self, *args)

    def supprev(self, *args) -> "nodeidx_t":
        r"""
        supprev(self, cur, tag=stag) -> nodeidx_t
        Get previous existing element of supval array.

        @param cur: (C++: nodeidx_t) current index
        @param tag: (C++: uchar) tag of array
        @return: index of the previous existing element of supval array BADNODE if no
                 more supval array elements exist
        """
        return _ida_netnode.netnode_supprev(self, *args)

    def supshift(self, *args) -> "size_t":
        r"""
        supshift(self, _from, to, size, tag=stag) -> size_t
        Shift the supval array elements. Moves the array elements at (from..from+size)
        to (to..to+size)

        @param from: (C++: nodeidx_t)
        @param to: (C++: nodeidx_t)
        @param size: (C++: nodeidx_t)
        @param tag: (C++: uchar)
        @return: number of shifted elements
        """
        return _ida_netnode.netnode_supshift(self, *args)

    def supval_idx8(self, *args) -> "ssize_t":
        r"""
        supval_idx8(self, alt, tag) -> ssize_t

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_supval_idx8(self, *args)

    def supstr_idx8(self, *args) -> "ssize_t":
        r"""
        supstr_idx8(self, alt, tag) -> ssize_t

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_supstr_idx8(self, *args)

    def supset_idx8(self, *args) -> "bool":
        r"""
        supset_idx8(self, alt, value, tag) -> bool

        @param alt: uchar
        @param value: void const *
        @param tag: uchar
        """
        return _ida_netnode.netnode_supset_idx8(self, *args)

    def supdel_idx8(self, *args) -> "bool":
        r"""
        supdel_idx8(self, alt, tag) -> bool

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_supdel_idx8(self, *args)

    def lower_bound_idx8(self, *args) -> "nodeidx_t":
        r"""
        lower_bound_idx8(self, alt, tag) -> nodeidx_t

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_lower_bound_idx8(self, *args)

    def supfirst_idx8(self, *args) -> "nodeidx_t":
        r"""
        supfirst_idx8(self, tag) -> nodeidx_t

        @param tag: uchar
        """
        return _ida_netnode.netnode_supfirst_idx8(self, *args)

    def supnext_idx8(self, *args) -> "nodeidx_t":
        r"""
        supnext_idx8(self, alt, tag) -> nodeidx_t

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_supnext_idx8(self, *args)

    def suplast_idx8(self, *args) -> "nodeidx_t":
        r"""
        suplast_idx8(self, tag) -> nodeidx_t

        @param tag: uchar
        """
        return _ida_netnode.netnode_suplast_idx8(self, *args)

    def supprev_idx8(self, *args) -> "nodeidx_t":
        r"""
        supprev_idx8(self, alt, tag) -> nodeidx_t

        @param alt: uchar
        @param tag: uchar
        """
        return _ida_netnode.netnode_supprev_idx8(self, *args)

    def supdel(self, *args) -> "bool":
        r"""
        supdel(self, alt, tag=stag) -> bool
        Delete all elements of supval array. This function may be applied to 32-bit and
        8-bit supval arrays. This function deletes the whole supval array.

        @param alt: nodeidx_t
        @param tag: uchar

        @return: success
        supdel(self) -> bool
        """
        return _ida_netnode.netnode_supdel(self, *args)

    def supdel_all(self, *args) -> "bool":
        r"""
        supdel_all(self, tag) -> bool
        Delete all elements of the specified supval array. This function may be applied
        to 32-bit and 8-bit supval arrays. This function deletes the whole supval array.

        @param tag: (C++: uchar)
        @return: success
        """
        return _ida_netnode.netnode_supdel_all(self, *args)

    def supdel_range(self, *args) -> "int":
        r"""
        supdel_range(self, idx1, idx2, tag) -> int
        Delete range of elements in the specified supval array. Elements in range [idx1,
        idx2) will be deleted.
        @note: This function can also be used to delete a range of altval elements

        @param idx1: (C++: nodeidx_t) first element to delete
        @param idx2: (C++: nodeidx_t) last element to delete + 1
        @param tag: (C++: uchar) tag of array
        @return: number of deleted elements
        """
        return _ida_netnode.netnode_supdel_range(self, *args)

    def supdel_range_idx8(self, *args) -> "int":
        r"""
        supdel_range_idx8(self, idx1, idx2, tag) -> int
        Same as above, but accepts 8-bit indexes.

        @param idx1: (C++: uchar)
        @param idx2: (C++: uchar)
        @param tag: (C++: uchar)
        """
        return _ida_netnode.netnode_supdel_range_idx8(self, *args)

    def hashval(self, *args) -> "ssize_t":
        r"""
        hashval(self, idx, tag=htag) -> ssize_t
        Get value of the specified hash element.

        @param idx: (C++: const char *) index into hash
        @param tag: (C++: uchar) tag of hash. Default: htag
        @return: -1 if element doesn't exist or idx is nullptr. otherwise returns the
                 value size in bytes
        """
        return _ida_netnode.netnode_hashval(self, *args)

    def hashstr(self, *args) -> "ssize_t":
        r"""
        hashstr(self, idx, tag=htag) -> ssize_t
        @see: hashstr(qstring *buf, const char *idx, uchar tag=htag) const

        @param idx: (C++: const char *) char const *
        @param tag: (C++: uchar)
        """
        return _ida_netnode.netnode_hashstr(self, *args)

    def hashval_long(self, *args) -> "nodeidx_t":
        r"""
        hashval_long(self, idx, tag=htag) -> nodeidx_t
        Get value of the specified hash element.

        @param idx: (C++: const char *) index into hash
        @param tag: (C++: uchar) tag of hash. Default: htag
        @return: value of hash element (it should be set using hashset(nodeidx_t)), 0 if
                 the element does not exist
        """
        return _ida_netnode.netnode_hashval_long(self, *args)

    def hashset(self, *args) -> "bool":
        r"""
        hashset(self, idx, value, tag=htag) -> bool
        Set value of hash element to long value.

        @param idx: (C++: const char *) index into hash
        @param value: (C++: nodeidx_t) new value of hash element
        @param tag: (C++: uchar) tag of hash. Default: htag
        @retval 1: ok
        @retval 0: should not occur - indicates internal error
        """
        return _ida_netnode.netnode_hashset(self, *args)

    def hashset_idx(self, *args) -> "bool":
        r"""
        hashset_idx(self, idx, value, tag=htag) -> bool

        @param idx: char const *
        @param value: nodeidx_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_hashset_idx(self, *args)

    def hashdel(self, *args) -> "bool":
        r"""
        hashdel(self, idx, tag=htag) -> bool
        Delete hash element.

        @param idx: (C++: const char *) index into hash
        @param tag: (C++: uchar) tag of hash. Default: htag
        @retval true: deleted
        @retval false: element does not exist
        """
        return _ida_netnode.netnode_hashdel(self, *args)

    def hashfirst(self, *args) -> "ssize_t":
        r"""
        hashfirst(self, tag=htag) -> ssize_t
        @see: hashfirst(qstring *buf, uchar tag=htag) const

        @param tag: (C++: uchar)
        """
        return _ida_netnode.netnode_hashfirst(self, *args)

    def hashnext(self, *args) -> "ssize_t":
        r"""
        hashnext(self, idx, tag=htag) -> ssize_t
        @see: hashnext(qstring *buf, const char *idx, uchar tag=htag) const

        @param idx: (C++: const char *) char const *
        @param tag: (C++: uchar)
        """
        return _ida_netnode.netnode_hashnext(self, *args)

    def hashlast(self, *args) -> "ssize_t":
        r"""
        hashlast(self, tag=htag) -> ssize_t
        @see: hashlast(qstring *buf, uchar tag=htag) const

        @param tag: (C++: uchar)
        """
        return _ida_netnode.netnode_hashlast(self, *args)

    def hashprev(self, *args) -> "ssize_t":
        r"""
        hashprev(self, idx, tag=htag) -> ssize_t
        @see: hashprev(qstring *buf, const char *idx, uchar tag=htag) const

        @param idx: (C++: const char *) char const *
        @param tag: (C++: uchar)
        """
        return _ida_netnode.netnode_hashprev(self, *args)

    def hashdel_all(self, *args) -> "bool":
        r"""
        hashdel_all(self, tag=htag) -> bool
        Delete all elements of hash. This function deletes the whole hash.

        @param tag: (C++: uchar) tag of hash. Default: htag
        @return: success
        """
        return _ida_netnode.netnode_hashdel_all(self, *args)

    def blobsize(self, *args) -> "size_t":
        r"""
        blobsize(self, _start, tag) -> size_t
        Get size of blob.

        @param _start: (C++: nodeidx_t) index of the first supval element used to store blob
        @param tag: (C++: uchar) tag of supval array
        @return: number of bytes required to store a blob
        """
        return _ida_netnode.netnode_blobsize(self, *args)

    def blobsize_ea(self, *args) -> "size_t":
        r"""
        blobsize_ea(self, ea, tag) -> size_t

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_blobsize_ea(self, *args)

    def setblob(self, *args) -> "bool":
        r"""
        setblob(self, buf, _start, tag) -> bool
        Store a blob in a netnode.

        @param buf: (C++: const void *) pointer to blob to save
        @param _start: (C++: nodeidx_t)
        @param tag: (C++: uchar) tag of supval array
        @return: success
        """
        return _ida_netnode.netnode_setblob(self, *args)

    def setblob_ea(self, *args) -> "bool":
        r"""
        setblob_ea(self, buf, ea, tag) -> bool

        @param buf: void const *
        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_setblob_ea(self, *args)

    def delblob(self, *args) -> "int":
        r"""
        delblob(self, _start, tag) -> int
        Delete a blob.

        @param _start: (C++: nodeidx_t) index of the first supval element used to store blob
        @param tag: (C++: uchar) tag of supval array
        @return: number of deleted supvals
        """
        return _ida_netnode.netnode_delblob(self, *args)

    def delblob_ea(self, *args) -> "int":
        r"""
        delblob_ea(self, ea, tag) -> int

        @param ea: ea_t
        @param tag: uchar
        """
        return _ida_netnode.netnode_delblob_ea(self, *args)

    def blobshift(self, *args) -> "size_t":
        r"""
        blobshift(self, _from, to, size, tag) -> size_t
        Shift the blob array elements. Moves the array elements at (from..from+size) to
        (to..to+size)

        @param from: (C++: nodeidx_t)
        @param to: (C++: nodeidx_t)
        @param size: (C++: nodeidx_t)
        @param tag: (C++: uchar)
        @return: number of shifted elements
        """
        return _ida_netnode.netnode_blobshift(self, *args)

    def start(self, *args) -> "bool":
        r"""
        start(self) -> bool
        Get first netnode in the graph. Sets netnodenumber to the lowest existing
        number.
        @retval true: ok
        @retval false: graph is empty
        """
        return _ida_netnode.netnode_start(self, *args)

    def end(self, *args) -> "bool":
        r"""
        end(self) -> bool
        Get last netnode in the graph. Sets netnodenumber to the highest existing
        number.
        @retval true: ok
        @retval false: graph is empty
        """
        return _ida_netnode.netnode_end(self, *args)

    def next(self, *args) -> "bool":
        r"""
        next(self) -> bool
        Get next netnode in the graph. Sets netnodenumber to the next existing number
        @retval true: ok
        @retval false: no more netnodes
        """
        return _ida_netnode.netnode_next(self, *args)

    def prev(self, *args) -> "bool":
        r"""
        prev(self) -> bool
        Get prev netnode in the graph. Sets netnodenumber to the previous existing
        number
        @retval true: ok
        @retval false: no more netnodes
        """
        return _ida_netnode.netnode_prev(self, *args)

    def copyto(self, *args) -> "size_t":
        r"""
        copyto(self, destnode, count=1) -> size_t

        @param destnode: netnode
        @param count: nodeidx_t
        """
        return _ida_netnode.netnode_copyto(self, *args)

    def moveto(self, *args) -> "size_t":
        r"""
        moveto(self, destnode, count=1) -> size_t

        @param destnode: netnode
        @param count: nodeidx_t
        """
        return _ida_netnode.netnode_moveto(self, *args)

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, n) -> bool

        @param n: netnode &

        __eq__(self, x) -> bool

        @param x: nodeidx_t
        """
        return _ida_netnode.netnode___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, n) -> bool

        @param n: netnode &

        __ne__(self, x) -> bool

        @param x: nodeidx_t
        """
        return _ida_netnode.netnode___ne__(self, *args)

    def index(self, *args) -> "nodeidx_t":
        r"""
        index(self) -> nodeidx_t
        """
        return _ida_netnode.netnode_index(self, *args)

    def getblob(self, *args) -> "PyObject *":
        r"""
        getblob(self, start, tag) -> bytes or None
        Get blob from a netnode into a qstring* and make sure the string is null-
        terminated.

        @param start: nodeidx_t
        @param tag: (C++: uchar) tag of supval array
        @return: -1 if blob doesn't exist size of string (including terminating null)
                 otherwise
        """
        return _ida_netnode.netnode_getblob(self, *args)

    def getclob(self, *args) -> "PyObject *":
        r"""
        getclob(self, start, tag) -> str

        @param start: nodeidx_t
        @param tag: char
        """
        return _ida_netnode.netnode_getclob(self, *args)

    def getblob_ea(self, *args) -> "PyObject *":
        r"""
        getblob_ea(self, ea, tag) -> PyObject *

        @param ea: ea_t
        @param tag: char
        """
        return _ida_netnode.netnode_getblob_ea(self, *args)

    def hashstr_buf(self, *args) -> "PyObject *":
        r"""
        hashstr_buf(self, idx, tag=htag) -> PyObject *

        @param idx: char const *
        @param tag: char
        """
        return _ida_netnode.netnode_hashstr_buf(self, *args)

    def hashset_buf(self, *args) -> "bool":
        r"""
        hashset_buf(self, idx, py_str, tag=htag) -> bool

        @param idx: char const *
        @param py_str: PyObject *
        @param tag: char
        """
        return _ida_netnode.netnode_hashset_buf(self, *args)

    def supset(self, *args) -> "bool":
        r"""
        supset(self, alt, value, tag=stag) -> bool
        Set value of supval array element.

        @param alt: (C++: nodeidx_t) index into array of supvals
        @param value: (C++: const void *) pointer to supval value
        @param tag: (C++: uchar) tag of array
        @retval 1: ok
        @retval 0: should not occur - indicates internal error
        """
        return _ida_netnode.netnode_supset(self, *args)

    def supset_ea(self, *args) -> "bool":
        r"""
        supset_ea(self, ea, value, tag=stag) -> bool

        @param ea: ea_t
        @param value: void const *
        @param tag: uchar

        """
        return _ida_netnode.netnode_supset_ea(self, *args)
    __swig_destroy__ = _ida_netnode.delete_netnode

# Register netnode in _ida_netnode:
_ida_netnode.netnode_swigregister(netnode)
cvar = _ida_netnode.cvar
MAXNAMESIZE = cvar.MAXNAMESIZE
r"""
Maximum length of a netnode name. WILL BE REMOVED IN THE FUTURE.
"""
MAX_NODENAME_SIZE = cvar.MAX_NODENAME_SIZE
r"""
Maximum length of a name. We permit names up to 32KB-1 bytes.
"""
MAXSPECSIZE = cvar.MAXSPECSIZE
r"""
Maximum length of strings or objects stored in a supval array element.
"""
atag = cvar.atag
r"""
Array of altvals.
"""
stag = cvar.stag
r"""
Array of supvals.
"""
htag = cvar.htag
r"""
Array of hashvals.
"""
vtag = cvar.vtag
r"""
Value of netnode.
"""
ntag = cvar.ntag
r"""
Name of netnode.
"""
ltag = cvar.ltag
r"""
Links between netnodes.
"""
NETMAP_IDX = cvar.NETMAP_IDX
NETMAP_VAL = cvar.NETMAP_VAL
NETMAP_STR = cvar.NETMAP_STR
NETMAP_X8 = cvar.NETMAP_X8
NETMAP_V8 = cvar.NETMAP_V8
NETMAP_VAL_NDX = cvar.NETMAP_VAL_NDX

def exist(*args) -> "bool":
    r"""
    exist(n) -> bool

    @param n: netnode const &
    """
    return _ida_netnode.exist(*args)

def netnode_exist(*args) -> "bool":
    r"""
    netnode_exist(_name) -> bool

    @param _name: char const *
    """
    return _ida_netnode.netnode_exist(*args)







"""
summary: enumerate patched bytes

description:
  Using the API to iterate over all the places in the file,
  that were patched using IDA.
"""

import ida_bytes
import ida_idaapi

# -------------------------------------------------------------------------
class patched_bytes_visitor(object):
    def __init__(self):
        self.skip = 0
        self.patch = 0

    def __call__(self, ea, fpos, o, v, cnt=()):
        if fpos == -1:
            self.skip += 1
            print("  ea: %x o: %x v: %x...skipped" % (ea, o, v))
        else:
            self.patch += 1
            print("  ea: %x fpos: %x o: %x v: %x" % (ea, fpos, o, v))
        return 0


# -------------------------------------------------------------------------
def main():
    print("Visiting all patched bytes:")
    v = patched_bytes_visitor()
    r = ida_bytes.visit_patched_bytes(0, ida_idaapi.BADADDR, v)
    if r != 0:
        print("visit_patched_bytes() returned %d" % r)
    else:
        print("Patched: %d Skipped: %d" % (v.patch, v.skip))


# -------------------------------------------------------------------------
if __name__ == '__main__':
    main()



"""
summary: showcases (a few of) the iterators available on a function

description:
  This demonstrates how to use some of the iterators available on the func_t type.

  This example will focus on:

    * `func_t[.__iter__]`: the default iterator; iterates on instructions
    * `func_t.data_items`: iterate on data items contained within a function
    * `func_t.head_items`: iterate on 'heads' (i.e., addresses containing
                           the start of an instruction, or a data item.
    * `func_t.addresses`: iterate on all addresses within function (code
                          and data, beginning of an item or not)

  Type `help(ida_funcs.func_t)` for a full list of iterators.

  In addition, one can use:

    * `func_tail_iterator_t`: iterate on all the chunks (including
                              the main one) of the function
    * `func_parent_iterator_t`: iterate on all the parent functions,
                                that include this chunk

keywords: funcs iterator
"""

import ida_bytes
import ida_kernwin
import ida_funcs
import ida_ua

class logger_t(object):

    class section_t(object):
        def __init__(self, logger, header):
            self.logger = logger
            self.logger.log(header)
        def __enter__(self):
            self.logger.indent += 2
            return self
        def __exit__(self, tp, value, traceback):
            self.logger.indent -= 2
            if value:
                return False # Re-raise

    def __init__(self):
        self.indent = 0

    def log(self, *args):
        print("  " * self.indent + "".join(args))

    def log_ea(self, ea):
        F = ida_bytes.get_flags(ea)
        parts = ["0x%08x" % ea, ": "]
        if ida_bytes.is_code(F):
            parts.append("instruction (%s)" % ida_ua.print_insn_mnem(ea))
        if ida_bytes.is_data(F):
            parts.append("data")
        if ida_bytes.is_tail(F):
            parts.append("tail")
        if ida_bytes.is_unknown(F):
            parts.append("unknown")
        if ida_funcs.get_func(ea) != ida_funcs.get_fchunk(ea):
            parts.append(" (in function chunk)")
        self.log(*parts)

def main():
    # Get current ea
    ea = ida_kernwin.get_screen_ea()

    pfn = ida_funcs.get_func(ea)

    if pfn is None:
        print("No function defined at 0x%x" % ea)
        return

    func_name = ida_funcs.get_func_name(pfn.start_ea)
    logger = logger_t()
    logger.log("Function %s at 0x%x" % (func_name, ea))

    with logger_t.section_t(logger, "Code items:"):
        for item in pfn:
            logger.log_ea(item)

    with logger_t.section_t(logger, "'head' items:"):
        for item in pfn.head_items():
            logger.log_ea(item)

    with logger_t.section_t(logger, "Addresses:"):
        for item in pfn.addresses():
            logger.log_ea(item)

    with logger_t.section_t(logger, "Function chunks:"):
        for chunk in ida_funcs.func_tail_iterator_t(pfn):
            logger.log("%s chunk: 0x%08x..0x%08x" % (
                "Main" if chunk.start_ea == pfn.start_ea else "Tail",
                chunk.start_ea,
                chunk.end_ea))

if __name__ == '__main__':
    main()



"""
File I/O functions for IDA.

You should not use standard C file I/O functions in modules. Use functions from
this header, pro.h and fpro.h instead.

This file also declares a call_system() function."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_diskio
else:
    import _ida_diskio

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_diskio.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi


def idadir(*args) -> "char const *":
    r"""
    idadir(subdir) -> char const *
    Get IDA directory (if subdir==nullptr) or the specified subdirectory (see IDA
    subdirectories)

    @param subdir: (C++: const char *) char const *
    """
    return _ida_diskio.idadir(*args)

def getsysfile(*args) -> "char const *":
    r"""
    getsysfile(filename, subdir) -> str
    Search for IDA system file. This function searches for a file in:
    1. each directory specified by IDAUSR%
    2. ida directory [+ subdir] and returns the first match.

    @param filename: (C++: const char *) name of file to search
    @param subdir: (C++: const char *) if specified, the file is looked for in the specified
                   subdirectory of the ida directory first (see IDA subdirectories)
    @return: nullptr if not found, otherwise a pointer to full file name.
    """
    return _ida_diskio.getsysfile(*args)
CFG_SUBDIR = _ida_diskio.CFG_SUBDIR

IDC_SUBDIR = _ida_diskio.IDC_SUBDIR

IDS_SUBDIR = _ida_diskio.IDS_SUBDIR

IDP_SUBDIR = _ida_diskio.IDP_SUBDIR

LDR_SUBDIR = _ida_diskio.LDR_SUBDIR

SIG_SUBDIR = _ida_diskio.SIG_SUBDIR

TIL_SUBDIR = _ida_diskio.TIL_SUBDIR

PLG_SUBDIR = _ida_diskio.PLG_SUBDIR

THM_SUBDIR = _ida_diskio.THM_SUBDIR


def get_user_idadir(*args) -> "char const *":
    r"""
    get_user_idadir() -> char const *
    Get user ida related directory.
    - if $IDAUSR is defined:
    - the first element in $IDAUSR
    - else
    - default user directory ($HOME/.idapro or %APPDATA%Hex-Rays/IDA Pro)
    """
    return _ida_diskio.get_user_idadir(*args)

def get_ida_subdirs(*args) -> "qstrvec_t *":
    r"""
    get_ida_subdirs(subdir, flags=0) -> int
    Get list of directories in which to find a specific IDA resource (see IDA
    subdirectories). The order of the resulting list is as follows:
    - [$IDAUSR/subdir (0..N entries)]
    - $IDADIR/subdir

    @param subdir: (C++: const char *) name of the resource to list
    @param flags: (C++: int) Subdirectory modification flags bits
    @return: number of directories appended to 'dirs'
    """
    return _ida_diskio.get_ida_subdirs(*args)
IDA_SUBDIR_IDP = _ida_diskio.IDA_SUBDIR_IDP
r"""
append the processor name as a subdirectory
"""

IDA_SUBDIR_IDADIR_FIRST = _ida_diskio.IDA_SUBDIR_IDADIR_FIRST
r"""
$IDADIR/subdir will be first, not last
"""

IDA_SUBDIR_ONLY_EXISTING = _ida_diskio.IDA_SUBDIR_ONLY_EXISTING
r"""
only existing directories will be present
"""


def get_special_folder(*args) -> "size_t":
    r"""
    get_special_folder(csidl) -> str
    Get a folder location by CSIDL (see Common CSIDLs). Path should be of at least
    MAX_PATH size

    @param csidl: (C++: int)
    """
    return _ida_diskio.get_special_folder(*args)
CSIDL_APPDATA = _ida_diskio.CSIDL_APPDATA

CSIDL_LOCAL_APPDATA = _ida_diskio.CSIDL_LOCAL_APPDATA

CSIDL_PROGRAM_FILES = _ida_diskio.CSIDL_PROGRAM_FILES

CSIDL_PROGRAM_FILES_COMMON = _ida_diskio.CSIDL_PROGRAM_FILES_COMMON

CSIDL_PROGRAM_FILESX86 = _ida_diskio.CSIDL_PROGRAM_FILESX86

class file_enumerator_t(object):
    r"""
    Proxy of C++ file_enumerator_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def visit_file(self, *args) -> "int":
        r"""
        visit_file(self, file) -> int

        @param file: char const *
        """
        return _ida_diskio.file_enumerator_t_visit_file(self, *args)
    __swig_destroy__ = _ida_diskio.delete_file_enumerator_t

    def __init__(self, *args):
        r"""
        __init__(self) -> file_enumerator_t

        @param self: PyObject *
        """
        if self.__class__ == file_enumerator_t:
            _self = None
        else:
            _self = self
        _ida_diskio.file_enumerator_t_swiginit(self, _ida_diskio.new_file_enumerator_t(_self, *args))
    def __disown__(self):
        self.this.disown()
        _ida_diskio.disown_file_enumerator_t(self)
        return weakref.proxy(self)

# Register file_enumerator_t in _ida_diskio:
_ida_diskio.file_enumerator_t_swigregister(file_enumerator_t)


def enumerate_files2(*args) -> "int":
    r"""
    enumerate_files2(answer, answer_size, path, fname, fv) -> int

    @param answer: char *
    @param answer_size: size_t
    @param path: char const *
    @param fname: char const *
    @param fv: file_enumerator_t &
    """
    return _ida_diskio.enumerate_files2(*args)

def fopenWT(*args) -> "FILE *":
    r"""
    fopenWT(file) -> FILE *
    Open a new file for write in text mode, deny write. If a file exists, it will be
    removed.

    @param file: (C++: const char *) char const *
    @return: nullptr if failure
    """
    return _ida_diskio.fopenWT(*args)

def fopenWB(*args) -> "FILE *":
    r"""
    fopenWB(file) -> FILE *
    Open a new file for write in binary mode, deny read/write. If a file exists, it
    will be removed.

    @param file: (C++: const char *) char const *
    @return: nullptr if failure
    """
    return _ida_diskio.fopenWB(*args)

def fopenRT(*args) -> "FILE *":
    r"""
    fopenRT(file) -> FILE *
    Open a file for read in text mode, deny none.

    @param file: (C++: const char *) char const *
    @return: nullptr if failure
    """
    return _ida_diskio.fopenRT(*args)

def fopenRB(*args) -> "FILE *":
    r"""
    fopenRB(file) -> FILE *
    Open a file for read in binary mode, deny none.

    @param file: (C++: const char *) char const *
    @return: nullptr if failure
    """
    return _ida_diskio.fopenRB(*args)

def fopenM(*args) -> "FILE *":
    r"""
    fopenM(file) -> FILE *
    Open a file for read/write in binary mode, deny write.

    @param file: (C++: const char *) char const *
    @return: nullptr if failure
    """
    return _ida_diskio.fopenM(*args)

def fopenA(*args) -> "FILE *":
    r"""
    fopenA(file) -> FILE *
    Open a file for append in text mode, deny none.

    @param file: (C++: const char *) char const *
    @return: nullptr if failure
    """
    return _ida_diskio.fopenA(*args)
class ioports_fallback_t(object):
    r"""
    Proxy of C++ ioports_fallback_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def handle(self, *args) -> "bool":
        r"""
        handle(self, ports, line) -> bool

        @param ports: ioports_t const &
        @param line: char const *
        """
        return _ida_diskio.ioports_fallback_t_handle(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> ioports_fallback_t

        @param self: PyObject *
        """
        if self.__class__ == ioports_fallback_t:
            _self = None
        else:
            _self = self
        _ida_diskio.ioports_fallback_t_swiginit(self, _ida_diskio.new_ioports_fallback_t(_self, *args))
    __swig_destroy__ = _ida_diskio.delete_ioports_fallback_t
    def __disown__(self):
        self.this.disown()
        _ida_diskio.disown_ioports_fallback_t(self)
        return weakref.proxy(self)

# Register ioports_fallback_t in _ida_diskio:
_ida_diskio.ioports_fallback_t_swigregister(ioports_fallback_t)


def read_ioports2(*args) -> "ssize_t":
    r"""
    read_ioports2(ports, device, file, callback=None) -> ssize_t

    @param ports: ioports_t *
    @param device: qstring *
    @param file: char const *
    @param callback: ioports_fallback_t *
    """
    return _ida_diskio.read_ioports2(*args)
class choose_ioport_parser_t(object):
    r"""
    Proxy of C++ choose_ioport_parser_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def parse(self, *args) -> "bool":
        r"""
        parse(self, param, line) -> bool
        @retval true: and fill PARAM with a displayed string
        @retval false: and empty PARAM to skip the current device
        @retval false: and fill PARAM with an error message

        @param param: (C++: qstring *)
        @param line: (C++: const char *) char const *
        """
        return _ida_diskio.choose_ioport_parser_t_parse(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> choose_ioport_parser_t

        @param self: PyObject *
        """
        if self.__class__ == choose_ioport_parser_t:
            _self = None
        else:
            _self = self
        _ida_diskio.choose_ioport_parser_t_swiginit(self, _ida_diskio.new_choose_ioport_parser_t(_self, *args))
    __swig_destroy__ = _ida_diskio.delete_choose_ioport_parser_t
    def __disown__(self):
        self.this.disown()
        _ida_diskio.disown_choose_ioport_parser_t(self)
        return weakref.proxy(self)

# Register choose_ioport_parser_t in _ida_diskio:
_ida_diskio.choose_ioport_parser_t_swigregister(choose_ioport_parser_t)


def choose_ioport_device2(*args) -> "bool":
    r"""
    choose_ioport_device2(_device, file, parse_params) -> bool

    @param _device: qstring *
    @param file: char const *
    @param parse_params: choose_ioport_parser_t *
    """
    return _ida_diskio.choose_ioport_device2(*args)
LINPUT_NONE = _ida_diskio.LINPUT_NONE
r"""
invalid linput
"""

LINPUT_LOCAL = _ida_diskio.LINPUT_LOCAL
r"""
local file
"""

LINPUT_RFILE = _ida_diskio.LINPUT_RFILE
r"""
remote file ( debugger_t::open_file, debugger_t::read_file)
"""

LINPUT_PROCMEM = _ida_diskio.LINPUT_PROCMEM
r"""
debugged process memory (read_dbg_memory())
"""

LINPUT_GENERIC = _ida_diskio.LINPUT_GENERIC
r"""
generic linput
"""


def qlgetz(*args) -> "size_t":
    r"""
    qlgetz(li, fpos) -> str
    Read a zero-terminated string from the input. If fpos == -1 then no seek will be
    performed.

    @param li: (C++: linput_t *)
    @param fpos: (C++: int64)
    """
    return _ida_diskio.qlgetz(*args)

def open_linput(*args) -> "linput_t *":
    r"""
    open_linput(file, remote) -> linput_t *
    Open loader input.

    @param file: (C++: const char *) char const *
    @param remote: (C++: bool)
    """
    return _ida_diskio.open_linput(*args)
class generic_linput_t(object):
    r"""
    Proxy of C++ generic_linput_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    filesize = property(_ida_diskio.generic_linput_t_filesize_get, _ida_diskio.generic_linput_t_filesize_set, doc=r"""filesize""")
    r"""
    input file size
    """
    blocksize = property(_ida_diskio.generic_linput_t_blocksize_get, _ida_diskio.generic_linput_t_blocksize_set, doc=r"""blocksize""")
    r"""
    preferred block size to work with read/write sizes will be in multiples of this
    number. for example, 4096 is a nice value blocksize 0 means that the filesize is
    unknown. the internal cache will be disabled in this case. also, seeks from the
    file end will fail. blocksize=-1 means error.
    """

    def read(self, *args) -> "ssize_t":
        r"""
        read(self, off, buffer, nbytes) -> ssize_t

        @param off: qoff64_t
        @param buffer: void *
        @param nbytes: size_t
        """
        return _ida_diskio.generic_linput_t_read(self, *args)
    __swig_destroy__ = _ida_diskio.delete_generic_linput_t

# Register generic_linput_t in _ida_diskio:
_ida_diskio.generic_linput_t_swigregister(generic_linput_t)


def create_generic_linput(*args) -> "linput_t *":
    r"""
    create_generic_linput(gl) -> linput_t *
    Create a generic linput

    @param gl: (C++: generic_linput_t *) linput description. this object will be destroyed by close_linput()
               using "delete gl;"
    """
    return _ida_diskio.create_generic_linput(*args)

def create_memory_linput(*args) -> "linput_t *":
    r"""
    create_memory_linput(start, size) -> linput_t *
    Create a linput for process memory. This linput will use read_dbg_memory() to
    read data.

    @param start: (C++: ea_t) starting address of the input
    @param size: (C++: asize_t) size of the memory area to represent as linput if unknown, may be
                 passed as 0
    """
    return _ida_diskio.create_memory_linput(*args)

def get_linput_type(*args) -> "linput_type_t":
    r"""
    get_linput_type(li) -> linput_type_t
    Get linput type.

    @param li: (C++: linput_t *)
    """
    return _ida_diskio.get_linput_type(*args)

def eclose(*args) -> "void":
    r"""
    eclose(fp)

    @param fp: FILE *
    """
    return _ida_diskio.eclose(*args)

def enumerate_files(*args) -> "PyObject *":
    r"""
    enumerate_files(path, fname, callback) -> PyObject *
    Enumerate files in the specified directory while the callback returns 0.

    @param path: directory to enumerate files in
    @param fname: mask of file names to enumerate
    @param callback: a callable object that takes the filename as
                     its first argument and it returns 0 to continue
                     enumeration or non-zero to stop enumeration.
    @return:     None in case of script errors
        tuple(code, fname) : If the callback returns non-zero
    """
    return _ida_diskio.enumerate_files(*args)

def create_bytearray_linput(*args) -> "linput_t *":
    r"""
    create_bytearray_linput(s) -> linput_t *
    Trivial memory linput.

    @param s: qstring const &
    """
    return _ida_diskio.create_bytearray_linput(*args)

def close_linput(*args) -> "void":
    r"""
    close_linput(li)
    Close loader input.

    @param li: (C++: linput_t *)
    """
    return _ida_diskio.close_linput(*args)

#<pycode(py_diskio)>
#</pycode(py_diskio)>







"""
summary: superficially modify the decompilation output

description:
  modifies the decompilation output in a superficial manner,
  by removing some white spaces

  Note: this is rather crude, not quite "pythonic" code.
"""

import idautils
import idc
import ida_idaapi
import ida_hexrays
import ida_lines

def dbg(msg):
    #print(msg)
    pass

def is_cident_char(c):
    return c.isalnum() or c == '_'

def my_tag_skipcodes(l, storage):
    n = ida_lines.tag_skipcodes(l)
    dbg("Skipping %d chars ('%s')" % (n, l[0:n]))
    storage.append(l[0:n])
    return l[n:]

def remove_spaces(sl):
    dbg("*" * 80)
    l = sl.line
    out = []

    def push(c):
        dbg("Appending '%s'" % c)
        out.append(c)

    # skip initial spaces, do not compress them
    while True:
        l = my_tag_skipcodes(l, out)
        if not l:
            break
        c = l[0]
        if not c.isspace():
            break
        push(c)
        l = l[1:]

    # remove all spaces except in string and char constants
    delim = None # if not None, then we are skipping until 'delim'
    last = None # last seen character
    while True:
        # go until comments
        if l.startswith("//"):
            push(l)
            break
        dbg("-" * 60)
        nchars = ida_lines.tag_advance(l, 1)
        push(l[0:nchars])
        l = l[nchars:]
        l = my_tag_skipcodes(l, out)
        if not l:
            break
        c = l[0]
        dbg("c: '%s', last: '%s', l: '%s'" % (c, last, l))
        if delim:
            # we're inside a literal.
            if c == delim:
                delim = None # literal ended
        elif c == '"' or c == "'":
            delim = c # string/char literal started
        elif c.isspace():
            end = l.lstrip()
            nptr = my_tag_skipcodes(end, out)
            dbg("end: '%s', nptr: '%s'" % (end, nptr))
            # do not concatenate idents
            if not is_cident_char(last) or not is_cident_char(nptr[0]):
                l = end
                c = l[0] if l else ''
                dbg("new l: '%s'" % l)
        last = l[0] if l else ''

    sl.line = "".join(out)


class vds6_hooks_t(ida_hexrays.Hexrays_Hooks):
    def func_printed(self, cfunc):
        for sl in cfunc.get_pseudocode():
            remove_spaces(sl);
        return 0

# a plugin interface, boilerplate code
class my_plugin_t(ida_idaapi.plugin_t):
    flags = ida_idaapi.PLUGIN_HIDE
    wanted_name = "Hex-Rays space remover (IDAPython)"
    wanted_hotkey = ""
    comment = "Sample plugin6 for Hex-Rays decompiler"
    help = ""
    def init(self):
        if ida_hexrays.init_hexrays_plugin():
            self.vds6_hooks = vds6_hooks_t()
            self.vds6_hooks.hook()
            return ida_idaapi.PLUGIN_KEEP # keep us in the memory
    def term(self):
        self.vds6_hooks.unhook()
    def run(self, arg):
        pass

def PLUGIN_ENTRY():
    return my_plugin_t()



"""
Structure type management (assembly level types)"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_struct
else:
    import _ida_struct

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_struct.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

class dyn_member_ref_array(object):
    r"""
    Proxy of C++ dynamic_wrapped_array_t< member_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data = property(_ida_struct.dyn_member_ref_array_data_get, doc=r"""data""")
    count = property(_ida_struct.dyn_member_ref_array_count_get, doc=r"""count""")

    def __init__(self, *args):
        r"""
        __init__(self, _data, _count) -> dyn_member_ref_array

        @param _data: member_t *
        @param _count: size_t
        """
        _ida_struct.dyn_member_ref_array_swiginit(self, _ida_struct.new_dyn_member_ref_array(*args))

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_struct.dyn_member_ref_array___len__(self, *args)

    def __getitem__(self, *args) -> "member_t const &":
        r"""
        __getitem__(self, i) -> member_t

        @param i: size_t
        """
        return _ida_struct.dyn_member_ref_array___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: member_t const &
        """
        return _ida_struct.dyn_member_ref_array___setitem__(self, *args)

    __iter__ = ida_idaapi._bounded_getitem_iterator

    __swig_destroy__ = _ida_struct.delete_dyn_member_ref_array

# Register dyn_member_ref_array in _ida_struct:
_ida_struct.dyn_member_ref_array_swigregister(dyn_member_ref_array)

STRUC_SEPARATOR = _ida_struct.STRUC_SEPARATOR
r"""
structname.fieldname
"""

class member_t(object):
    r"""
    Proxy of C++ member_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    id = property(_ida_struct.member_t_id_get, _ida_struct.member_t_id_set, doc=r"""id""")
    r"""
    name(), cmt, rptcmt
    """
    soff = property(_ida_struct.member_t_soff_get, _ida_struct.member_t_soff_set, doc=r"""soff""")
    r"""
    start offset (for unions - number of the member 0..n)
    """
    eoff = property(_ida_struct.member_t_eoff_get, _ida_struct.member_t_eoff_set, doc=r"""eoff""")
    r"""
    end offset
    """
    flag = property(_ida_struct.member_t_flag_get, _ida_struct.member_t_flag_set, doc=r"""flag""")
    r"""
    type+representation bits
    """
    props = property(_ida_struct.member_t_props_get, _ida_struct.member_t_props_set, doc=r"""props""")
    r"""
    Struct member properties
    """

    def unimem(self, *args) -> "bool":
        r"""
        unimem(self) -> bool
        Is a member of a union?
        """
        return _ida_struct.member_t_unimem(self, *args)

    def has_union(self, *args) -> "bool":
        r"""
        has_union(self) -> bool
        Has members of type "union"?
        """
        return _ida_struct.member_t_has_union(self, *args)

    def by_til(self, *args) -> "bool":
        r"""
        by_til(self) -> bool
        Was the member created due to the type system?
        """
        return _ida_struct.member_t_by_til(self, *args)

    def has_ti(self, *args) -> "bool":
        r"""
        has_ti(self) -> bool
        Has type information?
        """
        return _ida_struct.member_t_has_ti(self, *args)

    def is_baseclass(self, *args) -> "bool":
        r"""
        is_baseclass(self) -> bool
        Is a base class member?
        """
        return _ida_struct.member_t_is_baseclass(self, *args)

    def is_dupname(self, *args) -> "bool":
        r"""
        is_dupname(self) -> bool
        Duplicate name was resolved during import?
        """
        return _ida_struct.member_t_is_dupname(self, *args)

    def is_destructor(self, *args) -> "bool":
        r"""
        is_destructor(self) -> bool
        Is a virtual destructor?
        """
        return _ida_struct.member_t_is_destructor(self, *args)

    def get_soff(self, *args) -> "ea_t":
        r"""
        get_soff(self) -> ea_t
        Get start offset (for unions - returns 0)
        """
        return _ida_struct.member_t_get_soff(self, *args)

    def get_size(self, *args) -> "asize_t":
        r"""
        get_size(self) -> asize_t
        Get member size.
        """
        return _ida_struct.member_t_get_size(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> member_t
        """
        _ida_struct.member_t_swiginit(self, _ida_struct.new_member_t(*args))
    __swig_destroy__ = _ida_struct.delete_member_t

# Register member_t in _ida_struct:
_ida_struct.member_t_swigregister(member_t)
MF_OK = _ida_struct.MF_OK
r"""
is the member ok? (always yes)
"""

MF_UNIMEM = _ida_struct.MF_UNIMEM
r"""
is a member of a union?
"""

MF_HASUNI = _ida_struct.MF_HASUNI
r"""
has members of type "union"?
"""

MF_BYTIL = _ida_struct.MF_BYTIL
r"""
the member was created due to the type system
"""

MF_HASTI = _ida_struct.MF_HASTI
r"""
has type information?
"""

MF_BASECLASS = _ida_struct.MF_BASECLASS
r"""
a special member representing base class
"""

MF_DTOR = _ida_struct.MF_DTOR
r"""
a special member representing destructor
"""

MF_DUPNAME = _ida_struct.MF_DUPNAME
r"""
duplicate name resolved with _N suffix (N==soff)
"""


class struc_t(object):
    r"""
    Proxy of C++ struc_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    id = property(_ida_struct.struc_t_id_get, _ida_struct.struc_t_id_set, doc=r"""id""")
    r"""
    struct id
    """
    memqty = property(_ida_struct.struc_t_memqty_get, _ida_struct.struc_t_memqty_set, doc=r"""memqty""")
    r"""
    number of members
    """
    members = property(_ida_struct.struc_t_members_get, _ida_struct.struc_t_members_set, doc=r"""members""")
    r"""
    only defined members are stored here. there may be gaps between members.
    """
    age = property(_ida_struct.struc_t_age_get, _ida_struct.struc_t_age_set, doc=r"""age""")
    r"""
    not used
    """
    props = property(_ida_struct.struc_t_props_get, _ida_struct.struc_t_props_set, doc=r"""props""")
    r"""
    Structure properties
    """

    def is_varstr(self, *args) -> "bool":
        r"""
        is_varstr(self) -> bool
        Is variable size structure?
        """
        return _ida_struct.struc_t_is_varstr(self, *args)

    def is_union(self, *args) -> "bool":
        r"""
        is_union(self) -> bool
        Is a union?
        """
        return _ida_struct.struc_t_is_union(self, *args)

    def has_union(self, *args) -> "bool":
        r"""
        has_union(self) -> bool
        Has members of type "union"?
        """
        return _ida_struct.struc_t_has_union(self, *args)

    def like_union(self, *args) -> "bool":
        r"""
        like_union(self) -> bool
        Is a union or contains members of type "union"?
        """
        return _ida_struct.struc_t_like_union(self, *args)

    def is_choosable(self, *args) -> "bool":
        r"""
        is_choosable(self) -> bool
        Is included in chooser list? Use set_struc_listed to change the listed status
        """
        return _ida_struct.struc_t_is_choosable(self, *args)

    def from_til(self, *args) -> "bool":
        r"""
        from_til(self) -> bool
        Does structure come from a type library?
        """
        return _ida_struct.struc_t_from_til(self, *args)

    def is_hidden(self, *args) -> "bool":
        r"""
        is_hidden(self) -> bool
        Is the structure collapsed? Use set_struc_hidden to change the hidden status
        """
        return _ida_struct.struc_t_is_hidden(self, *args)

    def is_frame(self, *args) -> "bool":
        r"""
        is_frame(self) -> bool
        Is this structure a function frame?
        """
        return _ida_struct.struc_t_is_frame(self, *args)

    def get_alignment(self, *args) -> "int":
        r"""
        get_alignment(self) -> int
        See SF_ALIGN.
        """
        return _ida_struct.struc_t_get_alignment(self, *args)

    def is_ghost(self, *args) -> "bool":
        r"""
        is_ghost(self) -> bool
        Is a ghost copy of a local type?
        """
        return _ida_struct.struc_t_is_ghost(self, *args)

    def is_synced(self, *args) -> "bool":
        r"""
        is_synced(self) -> bool
        Is synced with a local type?
        """
        return _ida_struct.struc_t_is_synced(self, *args)

    def is_mappedto(self, *args) -> "bool":
        r"""
        is_mappedto(self) -> bool
        Is mapped to a local type?
        """
        return _ida_struct.struc_t_is_mappedto(self, *args)

    def is_copyof(self, *args) -> "bool":
        r"""
        is_copyof(self) -> bool
        Is copied from a local type?
        """
        return _ida_struct.struc_t_is_copyof(self, *args)

    def set_alignment(self, *args) -> "void":
        r"""
        set_alignment(self, shift)
        Do not use; use set_struc_align()

        @param shift: (C++: int)
        """
        return _ida_struct.struc_t_set_alignment(self, *args)

    def get_last_member(self, *args) -> "member_t const *":
        r"""
        get_last_member(self) -> member_t
        """
        return _ida_struct.struc_t_get_last_member(self, *args)

    def set_ghost(self, *args) -> "void":
        r"""
        set_ghost(self, _is_ghost)

        @param _is_ghost: bool
        """
        return _ida_struct.struc_t_set_ghost(self, *args)

    def unsync(self, *args) -> "void":
        r"""
        unsync(self)
        """
        return _ida_struct.struc_t_unsync(self, *args)
    ordinal = property(_ida_struct.struc_t_ordinal_get, _ida_struct.struc_t_ordinal_set, doc=r"""ordinal""")
    r"""
    corresponding local type ordinal number
    """

    def __get_members__(self, *args) -> "dynamic_wrapped_array_t< member_t >":
        r"""
        __get_members__(self) -> dyn_member_ref_array
        """
        return _ida_struct.struc_t___get_members__(self, *args)

    members = property(__get_members__)
    def get_member(self, index):
        return self.members[index]

    __swig_destroy__ = _ida_struct.delete_struc_t

# Register struc_t in _ida_struct:
_ida_struct.struc_t_swigregister(struc_t)
SF_VAR = _ida_struct.SF_VAR
r"""
is variable size structure (varstruct)? a variable size structure is one with
the zero size last member. if the last member is a varstruct, then the current
structure is a varstruct too.
"""

SF_UNION = _ida_struct.SF_UNION
r"""
is a union? varunions are prohibited!
"""

SF_HASUNI = _ida_struct.SF_HASUNI
r"""
has members of type "union"?
"""

SF_NOLIST = _ida_struct.SF_NOLIST
r"""
don't include in the chooser list
"""

SF_TYPLIB = _ida_struct.SF_TYPLIB
r"""
the structure comes from type library
"""

SF_HIDDEN = _ida_struct.SF_HIDDEN
r"""
the structure is collapsed
"""

SF_FRAME = _ida_struct.SF_FRAME
r"""
the structure is a function frame
"""

SF_ALIGN = _ida_struct.SF_ALIGN
r"""
alignment (shift amount: 0..31)
"""

SF_GHOST = _ida_struct.SF_GHOST
r"""
ghost copy of a local type
"""



def get_struc_qty(*args) -> "size_t":
    r"""
    get_struc_qty() -> size_t
    Get number of known structures.
    """
    return _ida_struct.get_struc_qty(*args)

def get_first_struc_idx(*args) -> "uval_t":
    r"""
    get_first_struc_idx() -> uval_t
    Get index of first structure.

    @return: BADADDR if no known structures, 0 otherwise
    """
    return _ida_struct.get_first_struc_idx(*args)

def get_last_struc_idx(*args) -> "uval_t":
    r"""
    get_last_struc_idx() -> uval_t
    Get index of last structure.

    @return: BADADDR if no known structures, get_struc_qty()-1 otherwise
    """
    return _ida_struct.get_last_struc_idx(*args)

def get_prev_struc_idx(*args) -> "uval_t":
    r"""
    get_prev_struc_idx(idx) -> uval_t
    Get previous struct index.

    @param idx: (C++: uval_t)
    @return: BADADDR if resulting index is negative, otherwise idx - 1
    """
    return _ida_struct.get_prev_struc_idx(*args)

def get_next_struc_idx(*args) -> "uval_t":
    r"""
    get_next_struc_idx(idx) -> uval_t
    Get next struct index.

    @param idx: (C++: uval_t)
    @return: BADADDR if resulting index is out of bounds, otherwise idx++
    """
    return _ida_struct.get_next_struc_idx(*args)

def get_struc_idx(*args) -> "uval_t":
    r"""
    get_struc_idx(id) -> uval_t
    Get internal number of the structure.

    @param id: (C++: tid_t)
    """
    return _ida_struct.get_struc_idx(*args)

def get_struc_by_idx(*args) -> "tid_t":
    r"""
    get_struc_by_idx(idx) -> tid_t
    Get struct id by struct number.

    @param idx: (C++: uval_t)
    """
    return _ida_struct.get_struc_by_idx(*args)

def get_struc(*args) -> "struc_t *":
    r"""
    get_struc(id) -> struc_t
    Get pointer to struct type info.

    @param id: (C++: tid_t)
    """
    return _ida_struct.get_struc(*args)

def get_struc_id(*args) -> "tid_t":
    r"""
    get_struc_id(name) -> tid_t
    Get struct id by name.

    @param name: (C++: const char *) char const *
    """
    return _ida_struct.get_struc_id(*args)

def get_struc_name(*args) -> "qstring *":
    r"""
    get_struc_name(id, flags=0) -> str

    @param id: tid_t
    @param flags: int
    """
    return _ida_struct.get_struc_name(*args)
STRNFL_REGEX = _ida_struct.STRNFL_REGEX
r"""
apply regular expressions to beautify the name
"""


def get_struc_cmt(*args) -> "qstring *":
    r"""
    get_struc_cmt(id, repeatable) -> str
    Get struct comment.

    @param id: (C++: tid_t)
    @param repeatable: (C++: bool)
    """
    return _ida_struct.get_struc_cmt(*args)

def get_struc_size(*args) -> "asize_t":
    r"""
    get_struc_size(sptr) -> asize_t
    Get struct size (also see get_struc_size(const struc_t *))

    @param sptr: struc_t const *

    get_struc_size(id) -> asize_t

    @param id: tid_t
    """
    return _ida_struct.get_struc_size(*args)

def get_struc_prev_offset(*args) -> "ea_t":
    r"""
    get_struc_prev_offset(sptr, offset) -> ea_t
    Get offset of member with largest offset less than 'offset'.

    @param sptr: (C++: const struc_t *) struc_t const *
    @param offset: (C++: ea_t)
    @return: BADADDR if no prev offset
    """
    return _ida_struct.get_struc_prev_offset(*args)

def get_struc_next_offset(*args) -> "ea_t":
    r"""
    get_struc_next_offset(sptr, offset) -> ea_t
    Get offset of member with smallest offset larger than 'offset'.

    @param sptr: (C++: const struc_t *) struc_t const *
    @param offset: (C++: ea_t)
    @return: BADADDR if no next offset
    """
    return _ida_struct.get_struc_next_offset(*args)

def get_struc_last_offset(*args) -> "ea_t":
    r"""
    get_struc_last_offset(sptr) -> ea_t
    Get offset of last member.

    @param sptr: (C++: const struc_t *) struc_t const *
    @return: BADADDR if memqty == 0
    """
    return _ida_struct.get_struc_last_offset(*args)

def get_struc_first_offset(*args) -> "ea_t":
    r"""
    get_struc_first_offset(sptr) -> ea_t
    Get offset of first member.

    @param sptr: (C++: const struc_t *) struc_t const *
    @return: BADADDR if memqty == 0
    """
    return _ida_struct.get_struc_first_offset(*args)

def get_max_offset(*args) -> "ea_t":
    r"""
    get_max_offset(sptr) -> ea_t
    For unions: returns number of members, for structs: returns size of structure.

    @param sptr: (C++: struc_t *)
    """
    return _ida_struct.get_max_offset(*args)

def is_varstr(*args) -> "bool":
    r"""
    is_varstr(id) -> bool
    Is variable size structure?

    @param id: (C++: tid_t)
    """
    return _ida_struct.is_varstr(*args)

def is_union(*args) -> "bool":
    r"""
    is_union(id) -> bool
    Is a union?

    @param id: (C++: tid_t)
    """
    return _ida_struct.is_union(*args)

def get_member_struc(*args) -> "struc_t *":
    r"""
    get_member_struc(fullname) -> struc_t
    Get containing structure of member by its full name "struct.field".

    @param fullname: (C++: const char *) char const *
    """
    return _ida_struct.get_member_struc(*args)

def get_sptr(*args) -> "struc_t *":
    r"""
    get_sptr(mptr) -> struc_t
    Get child struct if member is a struct.

    @param mptr: (C++: const member_t *) member_t const *
    """
    return _ida_struct.get_sptr(*args)

def get_member(*args) -> "member_t *":
    r"""
    get_member(sptr, offset) -> member_t
    Get member at given offset.

    @param sptr: (C++: const struc_t *) struc_t const *
    @param offset: (C++: asize_t)
    """
    return _ida_struct.get_member(*args)

def get_member_id(*args) -> "tid_t":
    r"""
    get_member_id(sptr, offset) -> tid_t
    Get member id at given offset.

    @param sptr: (C++: const struc_t *) struc_t const *
    @param offset: (C++: asize_t)
    """
    return _ida_struct.get_member_id(*args)

def get_member_by_name(*args) -> "member_t *":
    r"""
    get_member_by_name(sptr, membername) -> member_t
    Get a member by its name, like "field44".

    @param sptr: (C++: const struc_t *) struc_t const *
    @param membername: (C++: const char *) char const *
    """
    return _ida_struct.get_member_by_name(*args)

def get_member_by_fullname(*args) -> "struc_t **":
    r"""
    get_member_by_fullname(fullname) -> member_t
    Get a member by its fully qualified name, "struct.field".

    @param fullname: (C++: const char *) char const *
    """
    return _ida_struct.get_member_by_fullname(*args)

def get_member_fullname(*args) -> "qstring *":
    r"""
    get_member_fullname(mid) -> str
    Get a member's fully qualified name, "struct.field".

    @param mid: (C++: tid_t)
    """
    return _ida_struct.get_member_fullname(*args)

def get_member_name(*args) -> "qstring *":
    r"""
    get_member_name(mid) -> str

    @param mid: tid_t
    """
    return _ida_struct.get_member_name(*args)

def get_member_cmt(*args) -> "qstring *":
    r"""
    get_member_cmt(mid, repeatable) -> str
    Get comment of structure member.

    @param mid: (C++: tid_t)
    @param repeatable: (C++: bool)
    """
    return _ida_struct.get_member_cmt(*args)

def get_member_size(*args) -> "asize_t":
    r"""
    get_member_size(NONNULL_mptr) -> asize_t
    Get size of structure member. May return 0 for the last member of varstruct. For
    union members, returns member_t::eoff.

    @param NONNULL_mptr: (C++: const member_t *) member_t const *
    """
    return _ida_struct.get_member_size(*args)

def is_varmember(*args) -> "bool":
    r"""
    is_varmember(mptr) -> bool
    Is variable size member?

    @param mptr: (C++: const member_t *) member_t const *
    """
    return _ida_struct.is_varmember(*args)

def get_best_fit_member(*args) -> "member_t *":
    r"""
    get_best_fit_member(sptr, offset) -> member_t
    Get member that is most likely referenced by the specified offset. Useful for
    offsets > sizeof(struct).

    @param sptr: (C++: const struc_t *) struc_t const *
    @param offset: (C++: asize_t)
    """
    return _ida_struct.get_best_fit_member(*args)

def get_next_member_idx(*args) -> "ssize_t":
    r"""
    get_next_member_idx(sptr, off) -> ssize_t
    Get the next member idx, if it does not exist, return -1.

    @param sptr: (C++: const struc_t *) struc_t const *
    @param off: (C++: asize_t)
    """
    return _ida_struct.get_next_member_idx(*args)

def get_prev_member_idx(*args) -> "ssize_t":
    r"""
    get_prev_member_idx(sptr, off) -> ssize_t
    Get the prev member idx, if it does not exist, return -1.

    @param sptr: (C++: const struc_t *) struc_t const *
    @param off: (C++: asize_t)
    """
    return _ida_struct.get_prev_member_idx(*args)

def add_struc(*args) -> "tid_t":
    r"""
    add_struc(idx, name, is_union=False) -> tid_t
    Create a structure type. if idx==BADADDR then add as the last idx. if
    name==nullptr then a name will be generated "struct_%d".

    @param idx: (C++: uval_t)
    @param name: (C++: const char *) char const *
    @param is_union: (C++: bool)
    """
    return _ida_struct.add_struc(*args)

def del_struc(*args) -> "bool":
    r"""
    del_struc(sptr) -> bool
    Delete a structure type.

    @param sptr: (C++: struc_t *)
    """
    return _ida_struct.del_struc(*args)

def set_struc_idx(*args) -> "bool":
    r"""
    set_struc_idx(sptr, idx) -> bool
    Set internal number of struct. Also see get_struc_idx(), get_struc_by_idx().

    @param sptr: (C++: const struc_t *) struc_t const *
    @param idx: (C++: uval_t)
    """
    return _ida_struct.set_struc_idx(*args)

def set_struc_align(*args) -> "bool":
    r"""
    set_struc_align(sptr, shift) -> bool
    Set structure alignment (SF_ALIGN)

    @param sptr: (C++: struc_t *)
    @param shift: (C++: int)
    """
    return _ida_struct.set_struc_align(*args)

def set_struc_name(*args) -> "bool":
    r"""
    set_struc_name(id, name) -> bool
    Set structure name.

    @param id: (C++: tid_t)
    @param name: (C++: const char *) char const *
    """
    return _ida_struct.set_struc_name(*args)

def set_struc_cmt(*args) -> "bool":
    r"""
    set_struc_cmt(id, cmt, repeatable) -> bool
    Set structure comment.

    @param id: (C++: tid_t)
    @param cmt: (C++: const char *) char const *
    @param repeatable: (C++: bool)
    """
    return _ida_struct.set_struc_cmt(*args)
STRUC_ERROR_MEMBER_OK = _ida_struct.STRUC_ERROR_MEMBER_OK
r"""
success
"""

STRUC_ERROR_MEMBER_NAME = _ida_struct.STRUC_ERROR_MEMBER_NAME
r"""
already has member with this name (bad name)
"""

STRUC_ERROR_MEMBER_OFFSET = _ida_struct.STRUC_ERROR_MEMBER_OFFSET
r"""
already has member at this offset
"""

STRUC_ERROR_MEMBER_SIZE = _ida_struct.STRUC_ERROR_MEMBER_SIZE
r"""
bad number of bytes or bad sizeof(type)
"""

STRUC_ERROR_MEMBER_TINFO = _ida_struct.STRUC_ERROR_MEMBER_TINFO
r"""
bad typeid parameter
"""

STRUC_ERROR_MEMBER_STRUCT = _ida_struct.STRUC_ERROR_MEMBER_STRUCT
r"""
bad struct id (the 1st argument)
"""

STRUC_ERROR_MEMBER_UNIVAR = _ida_struct.STRUC_ERROR_MEMBER_UNIVAR
r"""
unions can't have variable sized members
"""

STRUC_ERROR_MEMBER_VARLAST = _ida_struct.STRUC_ERROR_MEMBER_VARLAST
r"""
variable sized member should be the last member in the structure
"""

STRUC_ERROR_MEMBER_NESTED = _ida_struct.STRUC_ERROR_MEMBER_NESTED
r"""
recursive structure nesting is forbidden
"""


def add_struc_member(*args) -> "struc_error_t":
    r"""
    add_struc_member(sptr, fieldname, offset, flag, mt, nbytes) -> struc_error_t
    Add member to existing structure.

    @param sptr: (C++: struc_t *) structure to modify
    @param fieldname: (C++: const char *) if nullptr, then "anonymous_#" name will be generated
    @param offset: (C++: ea_t) BADADDR means add to the end of structure
    @param flag: (C++: flags64_t) type + representation bits
    @param mt: (C++: const opinfo_t *) additional info about member type. must be present for structs,
               offsets, enums, strings, struct offsets.
    @param nbytes: (C++: asize_t) if == 0 then the structure will be a varstruct. in this case the
                   member should be the last member in the structure
    """
    return _ida_struct.add_struc_member(*args)

def del_struc_member(*args) -> "bool":
    r"""
    del_struc_member(sptr, offset) -> bool
    Delete member at given offset.

    @param sptr: (C++: struc_t *)
    @param offset: (C++: ea_t)
    """
    return _ida_struct.del_struc_member(*args)

def del_struc_members(*args) -> "int":
    r"""
    del_struc_members(sptr, off1, off2) -> int
    Delete members which occupy range of offsets (off1..off2).

    @param sptr: (C++: struc_t *)
    @param off1: (C++: ea_t)
    @param off2: (C++: ea_t)
    @return: number of deleted members or -1 on error
    """
    return _ida_struct.del_struc_members(*args)

def set_member_name(*args) -> "bool":
    r"""
    set_member_name(sptr, offset, name) -> bool
    Set name of member at given offset.

    @param sptr: (C++: struc_t *)
    @param offset: (C++: ea_t)
    @param name: (C++: const char *) char const *
    """
    return _ida_struct.set_member_name(*args)

def set_member_type(*args) -> "bool":
    r"""
    set_member_type(sptr, offset, flag, mt, nbytes) -> bool
    Set type of member at given offset (also see add_struc_member())

    @param sptr: (C++: struc_t *)
    @param offset: (C++: ea_t)
    @param flag: (C++: flags64_t)
    @param mt: (C++: const opinfo_t *) opinfo_t const *
    @param nbytes: (C++: asize_t)
    """
    return _ida_struct.set_member_type(*args)

def set_member_cmt(*args) -> "bool":
    r"""
    set_member_cmt(mptr, cmt, repeatable) -> bool
    Set member comment.

    @param mptr: (C++: member_t *)
    @param cmt: (C++: const char *) char const *
    @param repeatable: (C++: bool)
    """
    return _ida_struct.set_member_cmt(*args)

def expand_struc(*args) -> "bool":
    r"""
    expand_struc(sptr, offset, delta, recalc=True) -> bool
    Expand/Shrink structure type.

    @param sptr: (C++: struc_t *)
    @param offset: (C++: ea_t)
    @param delta: (C++: adiff_t)
    @param recalc: (C++: bool)
    """
    return _ida_struct.expand_struc(*args)

def save_struc(*args) -> "void":
    r"""
    save_struc(sptr, may_update_ltypes=True)
    Update struct information in the database (internal function)

    @param sptr: (C++: struc_t *)
    @param may_update_ltypes: (C++: bool)
    """
    return _ida_struct.save_struc(*args)

def set_struc_hidden(*args) -> "void":
    r"""
    set_struc_hidden(sptr, is_hidden)
    Hide/unhide a struct type.

    @param sptr: (C++: struc_t *)
    @param is_hidden: (C++: bool)
    """
    return _ida_struct.set_struc_hidden(*args)

def set_struc_listed(*args) -> "void":
    r"""
    set_struc_listed(sptr, is_listed)
    Add/remove a struct type from the struct list.

    @param sptr: (C++: struc_t *)
    @param is_listed: (C++: bool)
    """
    return _ida_struct.set_struc_listed(*args)
SMT_BADARG = _ida_struct.SMT_BADARG
r"""
bad parameters
"""

SMT_NOCOMPAT = _ida_struct.SMT_NOCOMPAT
r"""
the new type is not compatible with the old type
"""

SMT_WORSE = _ida_struct.SMT_WORSE
r"""
the new type is worse than the old type
"""

SMT_SIZE = _ida_struct.SMT_SIZE
r"""
the new type is incompatible with the member size
"""

SMT_ARRAY = _ida_struct.SMT_ARRAY
r"""
arrays are forbidden as function arguments
"""

SMT_OVERLAP = _ida_struct.SMT_OVERLAP
r"""
member would overlap with members that cannot be deleted
"""

SMT_FAILED = _ida_struct.SMT_FAILED
r"""
failed to set new member type
"""

SMT_OK = _ida_struct.SMT_OK

SMT_KEEP = _ida_struct.SMT_KEEP
r"""
no need to change the member type, the old type is better
"""


def get_member_tinfo(*args) -> "bool":
    r"""
    get_member_tinfo(tif, mptr) -> bool
    Get tinfo for given member.

    @param tif: (C++: tinfo_t *)
    @param mptr: (C++: const member_t *) member_t const *
    """
    return _ida_struct.get_member_tinfo(*args)

def del_member_tinfo(*args) -> "bool":
    r"""
    del_member_tinfo(sptr, mptr) -> bool
    Delete tinfo for given member.

    @param sptr: (C++: struc_t *)
    @param mptr: (C++: member_t *)
    """
    return _ida_struct.del_member_tinfo(*args)

def set_member_tinfo(*args) -> "smt_code_t":
    r"""
    set_member_tinfo(sptr, mptr, memoff, tif, flags) -> smt_code_t
    Set tinfo for given member.

    @param sptr: (C++: struc_t *) containing struct
    @param mptr: (C++: member_t *) target member
    @param memoff: (C++: uval_t) offset within member
    @param tif: (C++: const tinfo_t &) type info
    @param flags: (C++: int) Set member tinfo flags
    """
    return _ida_struct.set_member_tinfo(*args)
SET_MEMTI_MAY_DESTROY = _ida_struct.SET_MEMTI_MAY_DESTROY
r"""
may destroy other members
"""

SET_MEMTI_COMPATIBLE = _ida_struct.SET_MEMTI_COMPATIBLE
r"""
new type must be compatible with the old
"""

SET_MEMTI_FUNCARG = _ida_struct.SET_MEMTI_FUNCARG
r"""
mptr is function argument (cannot create arrays)
"""

SET_MEMTI_BYTIL = _ida_struct.SET_MEMTI_BYTIL
r"""
new type was created by the type subsystem
"""

SET_MEMTI_USERTI = _ida_struct.SET_MEMTI_USERTI
r"""
user-specified type
"""


def get_or_guess_member_tinfo(*args) -> "bool":
    r"""
    get_or_guess_member_tinfo(tif, mptr) -> bool
    Try to get tinfo for given member - if failed, generate a tinfo using
    information about the member id from the disassembly

    @param tif: (C++: tinfo_t *)
    @param mptr: (C++: const member_t *) member_t const *
    """
    return _ida_struct.get_or_guess_member_tinfo(*args)

def retrieve_member_info(*args) -> "opinfo_t *":
    r"""
    retrieve_member_info(buf, mptr) -> opinfo_t
    Get operand type info for member.

    @param buf: (C++: opinfo_t *)
    @param mptr: (C++: const member_t *) member_t const *
    """
    return _ida_struct.retrieve_member_info(*args)

def is_anonymous_member_name(*args) -> "bool":
    r"""
    is_anonymous_member_name(name) -> bool
    Is member name prefixed with "anonymous"?

    @param name: (C++: const char *) char const *
    """
    return _ida_struct.is_anonymous_member_name(*args)

def is_dummy_member_name(*args) -> "bool":
    r"""
    is_dummy_member_name(name) -> bool
    Is member name an auto-generated name?

    @param name: (C++: const char *) char const *
    """
    return _ida_struct.is_dummy_member_name(*args)

def get_member_by_id(*args) -> "qstring *, struc_t **":
    r"""
    get_member_by_id(mid) -> member_t
    Check if the specified member id points to a struct member. convenience
    function.

    @param mid: (C++: tid_t)
    """
    return _ida_struct.get_member_by_id(*args)

def is_member_id(*args) -> "bool":
    r"""
    is_member_id(mid) -> bool
    Is a member id?

    @param mid: (C++: tid_t)
    """
    return _ida_struct.is_member_id(*args)

def is_special_member(*args) -> "bool":
    r"""
    is_special_member(id) -> bool
    Is a special member with the name beginning with ' '?

    @param id: (C++: tid_t)
    """
    return _ida_struct.is_special_member(*args)
class struct_field_visitor_t(object):
    r"""
    Proxy of C++ struct_field_visitor_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def visit_field(self, *args) -> "int":
        r"""
        visit_field(self, sptr, mptr) -> int

        @param sptr: struc_t *
        @param mptr: member_t *
        """
        return _ida_struct.struct_field_visitor_t_visit_field(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> struct_field_visitor_t

        @param self: PyObject *
        """
        if self.__class__ == struct_field_visitor_t:
            _self = None
        else:
            _self = self
        _ida_struct.struct_field_visitor_t_swiginit(self, _ida_struct.new_struct_field_visitor_t(_self, *args))
    __swig_destroy__ = _ida_struct.delete_struct_field_visitor_t
    def __disown__(self):
        self.this.disown()
        _ida_struct.disown_struct_field_visitor_t(self)
        return weakref.proxy(self)

# Register struct_field_visitor_t in _ida_struct:
_ida_struct.struct_field_visitor_t_swigregister(struct_field_visitor_t)


def visit_stroff_fields(*args) -> "adiff_t *":
    r"""
    visit_stroff_fields(sfv, path, disp, appzero) -> flags64_t
    Visit structure fields in a stroff expression or in a reference to a struct data
    variable. This function can be used to enumerate all components of an expression
    like 'a.b.c'.

    @param sfv: (C++: struct_field_visitor_t &) visitor object
    @param path: (C++: const tid_t *) struct path (path[0] contains the initial struct id)
    @param disp: (C++: adiff_t *) offset into structure
    @param appzero: (C++: bool) should visit field at offset zero?
    """
    return _ida_struct.visit_stroff_fields(*args)

def stroff_as_size(*args) -> "bool":
    r"""
    stroff_as_size(plen, sptr, value) -> bool
    Should display a structure offset expression as the structure size?

    @param plen: (C++: int)
    @param sptr: (C++: const struc_t *) struc_t const *
    @param value: (C++: asize_t)
    """
    return _ida_struct.stroff_as_size(*args)

def get_innermost_member(*args) -> "PyObject *":
    r"""
    get_innermost_member(sptr, offset) -> (member_t, struc_t, int)
    Get the innermost member at the given offset

    @param sptr: the starting structure
    @param offset: offset into the starting structure
    @return:     - None on failure
        - tuple(member_t, struct_t, offset)
          where member_t: a member in SPTR (it is not a structure),
                struct_t: the innermost structure,
                offset:   remaining offset into the returned member
    """
    return _ida_struct.get_innermost_member(*args)






"""
summary: iterate a cblock_t object

description:
  Using a `ida_hexrays.ctree_visitor_t`, search for
  `ida_hexrays.cit_block` instances and dump them.

author: EiNSTeiN_ (einstein@g3nius.org)
"""

import ida_hexrays

class cblock_visitor_t(ida_hexrays.ctree_visitor_t):

    def __init__(self):
        ida_hexrays.ctree_visitor_t.__init__(self, ida_hexrays.CV_FAST)

    def visit_insn(self, ins):
        if ins.op == ida_hexrays.cit_block:
            self.dump_block(ins.ea, ins.cblock)
        return 0

    def dump_block(self, ea, b):
        # iterate over all block instructions
        print("dumping block %x" % (ea, ))
        for ins in b:
            print("  %x: insn %s" % (ins.ea, ins.opname))


class vds7_hooks_t(ida_hexrays.Hexrays_Hooks):
    def maturity(self, cfunc, maturity):
        if maturity == ida_hexrays.CMAT_BUILT:
            cbv = cblock_visitor_t()
            cbv.apply_to(cfunc.body, None)
        return 0


if ida_hexrays.init_hexrays_plugin():
    vds7_hooks = vds7_hooks_t()
    vds7_hooks.hook()
else:
    print('cblock visitor: hexrays is not available.')



"""
summary: save, and then restore, positions in a listing

description:
  Shows how it is possible re-implement IDA's bookmark capability,
  using 2 custom actions: one action saves the current location,
  and the other restores it.

  Note that, contrary to actual bookmarks, this example:

    * remembers only 1 saved position
    * doesn't save that position in the IDB (and therefore cannot
      be restored if IDA is closed & reopened.)

keywords: listing, actions

see_also: jump_next_comment
"""

import ida_kernwin
import ida_moves

class listing_action_handler_t(ida_kernwin.action_handler_t):
    def update(self, ctx):
        is_listing = ctx.widget_type in [
            ida_kernwin.BWN_ENUMS,
            ida_kernwin.BWN_STRUCTS,
            ida_kernwin.BWN_DISASM,
            ida_kernwin.BWN_CUSTVIEW,
            ida_kernwin.BWN_PSEUDOCODE,
        ]
        return ida_kernwin.AST_ENABLE_FOR_WIDGET if is_listing else ida_kernwin.AST_DISABLE_FOR_WIDGET

class last_pos_t(object):
    def __init__(self, widget_title, lochist_entry):
        self.widget_title = widget_title
        self.lochist_entry = lochist_entry

last_pos = None

class save_position_ah_t(listing_action_handler_t):

    ACTION_NAME = "save_and_restore_listing_pos:save_position"
    ACTION_LABEL = "Save position"
    ACTION_SHORTCUT = "Ctrl+Shift+S"
    HELP_TEXT = "Press %s in a 'listing' widget such as 'IDA View-A', 'Enums', 'Structures', 'Pseudocode-A', ... to remember the position" % ACTION_SHORTCUT

    def activate(self, ctx):
        global last_pos
        e = ida_moves.lochist_entry_t()
        if ida_kernwin.get_custom_viewer_location(e, ctx.widget):
            last_pos = last_pos_t(ctx.widget_title, e)
        else:
            print("Failed to retrieve position")


class restore_position_ah_t(listing_action_handler_t):

    ACTION_NAME = "save_and_restore_listing_pos:restore_position"
    ACTION_LABEL = "Restore position"
    ACTION_SHORTCUT = "Ctrl+Shift+O"
    HELP_TEXT = "Press %s in a 'listing' widget such as 'IDA View-A', 'Enums', 'Structures', 'Pseudocode-A', ... to restore a previously-saved position" % ACTION_SHORTCUT

    def activate(self, ctx):
        global last_pos
        if last_pos:
            w = ida_kernwin.find_widget(last_pos.widget_title)
            if w:
                ida_kernwin.custom_viewer_jump(w, last_pos.lochist_entry)
            else:
                print("Widget \"%s\" not found" % last_pos.widget_title)
        else:
            print("No last position to restore")


klasses = [
    save_position_ah_t,
    restore_position_ah_t,
]

for klass in klasses:
    if ida_kernwin.unregister_action(klass.ACTION_NAME):
        print("Unregistered previously-registered action \"%s\"" % klass.ACTION_LABEL)

    if ida_kernwin.register_action(
            ida_kernwin.action_desc_t(
                klass.ACTION_NAME,
                klass.ACTION_LABEL,
                klass(),
                klass.ACTION_SHORTCUT)):
        print("Registered action \"%s\". %s" % (klass.ACTION_LABEL, klass.HELP_TEXT))




"""
summary: retrieve & dump current selection

description:
  Shows how to retrieve the selection from a listing
  widget ("IDA View-A", "Hex View-1", "Pseudocode-A", ...) as
  two "cursors", and from there retrieve (in fact, generate)
  the corresponding text.

  After running this script:

    * select some text in one of the listing widgets (i.e.,
      "IDA View-*", "Enums", "Structures", "Pseudocode-*")
    * press Ctrl+Shift+S to dump the selection

"""

import ida_kernwin
import ida_lines

class dump_selection_handler_t(ida_kernwin.action_handler_t):
    def activate(self, ctx):
        if ctx.has_flag(ida_kernwin.ACF_HAS_SELECTION):
            tp0, tp1 = ctx.cur_sel._from, ctx.cur_sel.to
            ud = ida_kernwin.get_viewer_user_data(ctx.widget)
            lnar = ida_kernwin.linearray_t(ud)
            lnar.set_place(tp0.at)
            lines = []
            while True:
                cur_place = lnar.get_place()
                first_line_ref = ida_kernwin.l_compare2(cur_place, tp0.at, ud)
                last_line_ref = ida_kernwin.l_compare2(cur_place, tp1.at, ud)
                if last_line_ref > 0: # beyond last line
                    break
                line = ida_lines.tag_remove(lnar.down())
                if last_line_ref == 0: # at last line
                    line = line[0:tp1.x]
                elif first_line_ref == 0: # at first line
                    line = ' ' * tp0.x + line[tp0.x:]
                lines.append(line)
            for line in lines:
                print(line)
        return 1

    def update(self, ctx):
        ok_widgets = [
            ida_kernwin.BWN_DISASM,
            ida_kernwin.BWN_STRUCTS,
            ida_kernwin.BWN_ENUMS,
            ida_kernwin.BWN_PSEUDOCODE,
        ]
        return ida_kernwin.AST_ENABLE_FOR_WIDGET \
            if ctx.widget_type in ok_widgets \
            else ida_kernwin.AST_DISABLE_FOR_WIDGET


# -----------------------------------------------------------------------
# create actions (and attach them to IDA View-A's context menu if possible)
ACTION_NAME = "dump_selection"
ACTION_SHORTCUT = "Ctrl+Shift+S"

if ida_kernwin.unregister_action(ACTION_NAME):
    print("Unregistered previously-registered action \"%s\"" % ACTION_NAME)

if ida_kernwin.register_action(
        ida_kernwin.action_desc_t(
            ACTION_NAME,
            "Dump selection",
            dump_selection_handler_t(),
            ACTION_SHORTCUT)):
    print("Registered action \"%s\"" % ACTION_NAME)
